# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.sites.models import Site
from django.urls import NoReverseMatch
from django.utils.translation import ugettext_lazy as _, override
from menus.menu_pool import menu_pool
from cms.utils.i18n import get_current_language, get_default_language
from cms.plugin_pool import plugin_pool
from aldryn_translation_tools.models import TranslatedAutoSlugifyMixin
from allink_core.core.loading import get_model


__all__ = [
    'AllinkDetailMixin',
    'AllinkTranslatedAutoSlugifyMixin',
    'AllinkInvalidatePlaceholderCacheMixin',
    'AllinkMetaTagMixin',
]


class AllinkDetailMixin:
    """
    Mixin for models with detail view

    - always used in combination with aldryn_translation_tools.models.TranslationHelperMixin
    """
    def get_detail_view(self, application_namespace=None):
        """
        :param application_namespace:
        an application_namespace, e.g 'news'
        - this is usually supplied, when calling from a app_content plugin with a specific apphook
        :return:
        fully qualified detail view identifier, e.g 'news:detail'
        """
        if application_namespace:
            return '{}:detail'.format(application_namespace)
        else:
            return '{}:detail'.format(self._meta.app_label)

    def get_absolute_url(self, language=None, application_namespace=None):
        """
        :param language:
        :param application_namespace:
        :return:

        """
        from django.urls import reverse
        if not language:
            language = get_current_language() or get_default_language()

        slug, language = self.known_translation_getter('slug', None, language_code=language)
        try:
            with override(language):
                return reverse(self.get_detail_view(application_namespace), kwargs={'slug': slug})
        except NoReverseMatch:
            # so we can spot this common problem also when not in DEBUG mode.
            return '/no_apphook_configured'


class AllinkTranslatedAutoSlugifyMixin(TranslatedAutoSlugifyMixin):
    """
    This is a TranslatableModel mixin that automatically generates a suitable
    slug for the object on save.
    If `slug_globally_unique` is True, then slugs will be required to be
    unique across all languages.
    If `slug_globally_unique` is False (default), then the strategy used here
    is that it is OK for two objects to use the same slug if the slugs are for
    different languages. So if this were used on an Article model, these would
    be valid:
        /en/pain -> Article in EN about physical discomfort
        /fr/pain -> Article in FR about bread
    Of course, this means that when resolving an object from its URL, care must
    be taken to factor in the language segment of the URL too.

    The allink version of this Mixin adds the functionality, that with each
    save we try to change the slug if it is a default slug. To a more specific one.

    This is especially useful for auto-generated categories, which get a default
    slug during creation.
    """

    def is_default_slug(self, slug):
        return slug and self.get_slug_default() in slug

    def save(self, **kwargs):
        slug = self._get_existing_slug()
        is_default = self.is_default_slug(slug)
        if not slug or self._slug_exists(slug) or is_default:
            new_slug = self.make_new_slug(slug=slug if not is_default else None)

            # we do not want to change a default slug to a new default slug
            if not(is_default and self.is_default_slug(new_slug)):
                setattr(self, self.slug_field_name, new_slug)
        # do not call direct superclass, it does the same (but less) again
        return super(TranslatedAutoSlugifyMixin, self).save(**kwargs)


class AllinkInvalidatePlaceholderCacheMixin:
    """
    This Mixin is used in combination with a CMS-Plugin. It makes sure that the placeholder cache keys get deleted,
    when the involved model instance is saved.

    The Plugin must define a attribute 'data_model', otherwise it won't get deleted.
    e.g:
        class HistoryPlugin(CMSPlugin):
            data_model = HistoryItem


    Only the placeholder caches get deleted, which contain relevant plugins.

    """

    def save(self, *args, **kwargs):
        super(AllinkInvalidatePlaceholderCacheMixin, self).save(*args, **kwargs)
        # we need to invalidate all placeholder cache keys,
        # where a plugin is placed that displayes data from this model
        # adapted from cms/models/pagemodel.py
        from cms.cache import invalidate_cms_page_cache
        invalidate_cms_page_cache()

        relevant_models = (self.__class__,) + self.__class__.__bases__
        relevant_plugin_classes = [x for x in plugin_pool.get_all_plugins() if
                                   hasattr(x.model, 'data_model') and x.model.data_model in relevant_models]

        # get all pages where a relevant plugin is placed
        for plugin_class in relevant_plugin_classes:
            for plugin in plugin_class.model.objects.all():
                for language_code, language in settings.LANGUAGES:
                    plugin.placeholder.clear_cache(language_code, site_id=getattr(plugin.page, 'site_id', None))

        # invalidate the menu for all sites
        for site in Site.objects.all():
            menu_pool.clear(site_id=site.id)


class AllinkMetaTagMixin:
    """
    Mixin for all relevant meta fields. Defines the fallback hierarchy.

    Used in combination with:

    AllinkTeaserFieldsModel,
    AllinkTeaserTranslatedFieldsModel,
    AllinkSEOFieldsModel,
    AllinkSEOTranslatedFieldsModel,

    """

    META_FIELD_FALLBACK_CONF = {
        'meta_image': [
            {'model': 'self', 'field': 'og_image', },
            {'model': 'self', 'field': 'teaser_image', },
            {'model': 'self', 'field': 'preview_image', },
            {'model': 'config.Config', 'field': 'default_og_image', },
        ],
        'meta_title': [
            {'model': 'self', 'field': 'og_title', },
            {'model': 'self', 'field': 'teaser_title', },
            {'model': 'self', 'field': 'title', },
        ],
        'meta_description': [
            {'model': 'self', 'field': 'og_description', },
            {'model': 'self', 'field': 'teaser_description', },
            {'model': 'self', 'field': 'lead', },
        ]
    }

    @property
    def field_fallback_conf(self):
        """
        the get_fallback function is interested in both META_FIELD_FALLBACK_CONF and TEASER_FIELD_FALLBACK_CONF
        but it is not guaranteed, that AllinkMetaTagMixin and AllinkTeaserMixin are used together.
        """
        try:
            return {**self.META_FIELD_FALLBACK_CONF, **self.TEASER_FIELD_FALLBACK_CONF}
        except AttributeError:
            return self.TEASER_FIELD_FALLBACK_CONF

    @property
    def meta_image_thumb(self):
        """
        We always want the meta image with a maximum width of 1200px. The ratio should be preserved.
        :return:
        image object
        """
        from allink_core.apps.config.utils import get_fallback, generate_meta_image_thumb
        meta_image = get_fallback(self, 'meta_image')
        return generate_meta_image_thumb(meta_image)

    @property
    def meta_page_title(self):
        from allink_core.apps.config.utils import get_fallback
        Config = get_model('config', 'Config')
        allink_config = Config.get_solo()

        base_title = ' | ' + getattr(allink_config, 'default_base_title', '')
        page_title = get_fallback(self, 'meta_title')

        return page_title + base_title

    @property
    def meta_dict(self):
        """
        :return:
        dict with all relevant meta fields
        e.g. used for seo template snippet: templatetags/allink_meta_og.html
        """
        from allink_core.apps.config.utils import get_fallback

        Config = get_model('config', 'Config')
        allink_config = Config.get_solo()

        meta_context = {
            'meta_page_title': self.meta_page_title,
            'meta_og_title': get_fallback(self, 'meta_title'),
            'meta_description': get_fallback(self, 'meta_description'),
            'meta_image_url': getattr(self.meta_image_thumb, 'url', ''),
            'google_site_verification': allink_config.google_site_verification,
        }
        return meta_context


class AllinkTeaserMixin:
    """
    Mixin for all relevant teaser fields. Defines the fallback hierarchy.

    Used in combination with:

    AllinkTeaserFieldsModel,
    AllinkTeaserTranslatedFieldsModel


    override TEASER_FIELD_FALLBACK_CONF on per app basis:
    ...
    """

    TEASER_LINK_TEXT = _('Read more')

    TEASER_FIELD_FALLBACK_CONF = {
        'teaser_image': [
            {'model': 'self', 'field': 'teaser_image', },
            {'model': 'self', 'field': 'preview_image', },
        ],
        'teaser_title': [
            {'model': 'self', 'field': 'teaser_title', },
            {'model': 'self', 'field': 'title', },
        ],
        'teaser_technical_title': [
            {'model': 'self', 'field': 'teaser_technical_title', },
        ],
        'teaser_description': [
            {'model': 'self', 'field': 'teaser_description', },
            {'model': 'self', 'field': 'lead', },
        ],
        'teaser_link_text': [
            {'model': 'self', 'field': 'teaser_link_text', },
            {'model': 'self', 'field': 'TEASER_LINK_TEXT', },
        ],
    }

    @property
    def field_fallback_conf(self):
        """
        the get_fallback function is interested in both META_FIELD_FALLBACK_CONF and TEASER_FIELD_FALLBACK_CONF
        but it is not guaranteed, that AllinkMetaTagMixin and AllinkTeaserMixin are used together.
        """
        try:
            return {**self.META_FIELD_FALLBACK_CONF, **self.TEASER_FIELD_FALLBACK_CONF}
        except AttributeError:
            return self.TEASER_FIELD_FALLBACK_CONF

    @property
    def teaser_dict(self):
        """
        :return:
        dict with all relevant teaser fields
        e.g. used for teaser templates e.g: allink_teaser/<<template_name>>/item.html
        """
        from allink_core.apps.config.utils import get_fallback

        teaser_context = {
            'teaser_title': get_fallback(self, 'teaser_title'),
            'teaser_technical_title': get_fallback(self, 'teaser_technical_title'),
            'teaser_description': get_fallback(self, 'teaser_description'),
            'teaser_image': get_fallback(self, 'teaser_image'),
            'teaser_link_text': get_fallback(self, 'teaser_link_text'),
            'teaser_link': self.get_absolute_url(),
        }
        return teaser_context



# -*- coding: utf-8 -*-

from django.db import models
from cms.models.pluginmodel import CMSPlugin
from django.contrib.postgres.fields import ArrayField


class AllinkSocialIconContainerPlugin(CMSPlugin):
    """
    A Container-Plugin for Social Icons
    """
    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )

    def __str__(self):
        return '{}'.format(str(self.pk))

    @property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        return ' '.join(css_classes)


class AllinkSocialIconPlugin(CMSPlugin):
    title = models.CharField(
        'Title',
        help_text='SEO text (not visible) e.g. Follow allink on Instagram',
        max_length=255,
        blank=True,
        null=True
    )
    icon = models.CharField(
        'Icon',
        max_length=50
    )
    link = models.URLField(
        'Link'
    )

    def __str__(self):
        return '{}'.format(self.icon)



from django.contrib import admin
from cms.utils.urlutils import admin_reverse
from cms.test_utils.testcases import CMSTestCase
from allink_core.core.test import PageApphookMixin
from ..cms_apps import NewsApphook
from ..admin import NewsAdmin
from .factories import NewsFactory


class NewsAdminTestCase(PageApphookMixin, CMSTestCase):

    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'

    apphook_object = NewsApphook


    def setUp(self):
        super().setUp()

        self.admin_user = self.get_superuser()
        self.entry_1 = NewsFactory()
        self.model_admin_1 = NewsAdmin(self.entry_1, admin.site)

    def test_all_languages_column(self):
        # Check that all the available languages appear in `all_translations`
        all_translations = self.model_admin_1.all_languages_column(self.entry_1)

        change_url = admin_reverse('news_news_change', args=[self.entry_1.id])

        self.assertTrue(change_url + '?language=en' in all_translations)
        self.assertTrue(change_url + '?language=de' in all_translations)
        self.assertTrue(change_url + '?language=fr' in all_translations)

    def test_change_list_view(self):
        with self.login_user_context(self.admin_user):
            response = self.client.get(admin_reverse('news_news_changelist'))
            self.assertEqual(response.status_code, 200)

    def test_change_view(self):
        with self.login_user_context(self.admin_user):
            response = self.client.get(admin_reverse('news_news_change', args=[self.entry_1.id]))
            self.assertEqual(response.status_code, 200)



# -*- coding: utf-8 -*-
from allink_core.core.sitemap import HrefLangSitemap
from .models import DummyApp


class DummyAppSitemap(HrefLangSitemap):
    changefreq = "never"
    priority = 0.5
    i18n = True

    def __init__(self, *args, **kwargs):
        self.namespace = kwargs.pop('namespace', None)
        super(DummyAppSitemap, self).__init__(*args, **kwargs)

    def items(self):
        return DummyApp.objects.translated()

    def lastmod(self, obj):
        return obj.modified



PAGE_META_FIELD_FALLBACK_CONF = {
    'meta_image': [
        {'model': 'self', 'field': 'allinkpageextension.og_image', },
        {'model': 'self', 'field': 'allinkpageextension.teaser_image', },
        {'model': 'config.Config', 'field': 'default_og_image', },
    ],
    'meta_title': [
        {'model': 'cms.Title', 'field': 'allinktitleextension.og_title', },
        {'model': 'cms.Title', 'field': 'allinktitleextension.teaser_title', },
        {'model': 'self', 'field': 'get_page_title', },
    ],
    'meta_description': [
        {'model': 'cms.Title', 'field': 'allinktitleextension.og_description', },
        {'model': 'cms.Title', 'field': 'allinktitleextension.teaser_description', },
    ],
}

PAGE_TEASER_FIELD_FALLBACK_CONF = {
    'teaser_image': [
        {'model': 'self', 'field': 'allinkpageextension.teaser_image', },
    ],
    'teaser_title': [
        {'model': 'cms.Title', 'field': 'allinktitleextension.teaser_title', },
        {'model': 'self', 'field': 'get_page_title', },
    ],
    'teaser_technical_title': [
        {'model': 'cms.Title', 'field': 'allinktitleextension.teaser_technical_title', },
    ],
    'teaser_description': [
        {'model': 'cms.Title', 'field': 'allinktitleextension.teaser_description', },
    ],
    'teaser_link_text': [
        {'model': 'cms.Title', 'field': 'allinktitleextension.teaser_link_text', },
        {'model': 'settings', 'field': 'TEASER_PAGE_LINK_TEXT', },
    ],
}

PAGE_FIELD_FALLBACK_CONF = {**PAGE_META_FIELD_FALLBACK_CONF, **PAGE_TEASER_FIELD_FALLBACK_CONF}



"""
use core:
no 'views.py' file needed
"""
from django.shortcuts import render
from allink_core.apps.dummy_app.views import DummyAppDetail as CoreDummyAppDetail


class DummyAppDetail(CoreDummyAppDetail):
    def render_to_response(self, context, **response_kwargs):
        if self.request.GET.get('softpage', None):
            context.update({'base_template': 'app_content/ajax_base.html'})
        return render(self.request, self.get_template_names(), context)


# example for adding a new view.
from django.views.generic import TemplateView
from allink_core.core.loading import get_model

DummyApp = get_model('dummy_app', 'DummyApp')


class DummyAppSomeNewView(TemplateView):
    template_name = 'dummy_app/some_template.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        item = DummyApp.objects.new().first()
        context.update({'something_more_usefull': item.title})
        return context



# -*- coding: utf-8 -*-
from allink_core.apps.people.abstract_models import BasePeople, BasePeopleTranslation, BasePeopleAppContentPlugin
from allink_core.core.loading import get_model
from allink_core.core.loading import is_model_registered

__all__ = []


if not is_model_registered('people', 'People'):
    class People(BasePeople):
        pass

    __all__.append('People')


if not is_model_registered('people', 'PeopleTranslation'):
    class PeopleTranslation(BasePeopleTranslation):
        pass

    __all__.append('PeopleTranslation')


if not is_model_registered('people', 'PeopleAppContentPlugin'):
    class PeopleAppContentPlugin(BasePeopleAppContentPlugin):
        data_model = get_model('people', 'People')

    __all__.append('PeopleAppContentPlugin')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django.conf import settings
from django.core.management import CommandError
from django.core.management.base import BaseCommand

from cms.api import copy_plugins_to_language
from cms.models import Page, StaticPlaceholder, EmptyTitle
from cms.utils import get_language_list
from cms.utils.copy_plugins import copy_plugins_to


class Command(BaseCommand):
    """
    e.g.:
    ./manage.py copylang --from-lang=en --to-lang=nl --tree="Home"
    ./manage.py copylang --from-lang=en --to-lang=nl --tree_id=36
    ./manage.py copylang --from-lang=en --to-lang=nl --tree_id=36 --force
    """
    help = ('Duplicate the cms content from one lang to another '
            '(to boot a new language) using draft pages. '
            'If a page is given with the --tree_id option or --tree option '
            'only this page and its children are copied.')

    def add_arguments(self, parser):
        parser.add_argument('--from-lang', action='store', dest='from_lang', required=True,
                            help='Language to copy the content from.')
        parser.add_argument('--to-lang', action='store', dest='to_lang', required=True,
                            help='Language to copy the content to.')
        parser.add_argument('--site', action='store', dest='site',
                            help='Site to work on.')
        parser.add_argument('--force', action='store_false', dest='only_empty', default=True,
                            help='If set content is copied even if destination language already '
                                 'has content.')
        parser.add_argument('--skip-content', action='store_false', dest='copy_content',
                            default=True, help='If set content is not copied, and the command '
                                               'will only create titles in the given language.')
        parser.add_argument('--tree_id', action='store', dest='page_id',
                            type=int, default=None,
                            help='Only copy the page tree starting with '
                                 'page with the given id.')
        parser.add_argument('--tree', action='store', dest='page',
                            type=str, default=None,
                            help='Only copy the page tree starting with '
                                 'the given title in "from" language.')

    def handle(self, *args, **options):
        verbose = options.get('verbosity') > 1
        only_empty = options.get('only_empty')
        copy_content = options.get('copy_content')
        from_lang = options.get('from_lang')
        to_lang = options.get('to_lang')
        try:
            site = int(options.get('site', None))
        except Exception:
            site = settings.SITE_ID

        try:
            assert from_lang in get_language_list(site)
            assert to_lang in get_language_list(site)
        except AssertionError:
            raise CommandError('Both languages have to be present in settings.LANGUAGES and settings.CMS_LANGUAGES')

        if options.get('page_id', None):
            head = Page.objects.on_site(site) \
                .drafts().get(id=options.get('page_id'))
            pages = [head] + list(head.children.drafts())
        else:
            pages = Page.objects.on_site(site).drafts()
            if options.get('page', None):
                head = None
                for page in pages:
                    if page.get_title_obj(from_lang).title == options.get('page'):
                        if head is not None:
                            raise CommandError('Page has to be unique on the site. Try the --tree_id option.')
                        head = page
                pages = [head] + list(head.children.drafts())

        for page in pages:
            # copy title
            if from_lang in page.get_languages():
                title = page.get_title_obj(to_lang, fallback=False)
                if isinstance(title, EmptyTitle):
                    title = page.get_title_obj(from_lang)
                    if verbose:
                        self.stdout.write('copying title %s from language %s\n' % (title.title, from_lang))
                    title.id = None
                    title.publisher_public_id = None
                    title.publisher_state = 0
                    title.language = to_lang
                    title.save()
                if verbose:
                    self.stdout.write('copying plugins for %s from %s\n' % (page.get_page_title(from_lang), from_lang))
                if copy_content:
                    # copy plugins using API
                    copy_plugins_to_language(page, from_lang, to_lang, only_empty)
            else:
                if verbose:
                    self.stdout.write('Skipping page %s, language %s not defined\n' % (
                        page.get_page_title(page.get_languages()[0]), from_lang))

        if copy_content:
            for static_placeholder in StaticPlaceholder.objects.all():
                plugin_list = []
                for plugin in static_placeholder.draft.get_plugins():
                    if plugin.language == from_lang:
                        plugin_list.append(plugin)

                if plugin_list:
                    if verbose:
                        self.stdout.write(
                            'copying plugins from static_placeholder "%s" in "%s" to "%s"\n' % (
                                static_placeholder.name, from_lang, to_lang)
                        )
                    copy_plugins_to(plugin_list, static_placeholder.draft, to_lang)

        self.stdout.write('all done')



# -*- coding: utf-8 -*-
from allink_core.core.models.managers import AllinkCategoryModelQuerySet


class AllinkLocationsQuerySet(AllinkCategoryModelQuerySet):
    pass


AllinkLocationsManager = AllinkLocationsQuerySet.as_manager



# -*- coding: utf-8 -*-s
from django.db import models
from django.utils.translation import ugettext_lazy as _
from cms.models.pluginmodel import CMSPlugin
from cms.models.fields import PageField
from adminsortable.fields import SortableForeignKey
from allink_core.core.models.fields import CMSPluginField


class AllinkPageChooserPlugin(CMSPlugin):
    cmsplugin_ptr = CMSPluginField()

    def copy_relations(self, oldinstance):
        self.allinkpage_set.all().delete()

        for allink_page in oldinstance.allinkpage_set.all():
            allink_page.pk = None
            allink_page.pagechooser_id = self.id
            allink_page.save()

    class Meta:
        app_label = 'allink_cms'


class AllinkPage(models.Model):
    pagechooser = SortableForeignKey(
        AllinkPageChooserPlugin,
        verbose_name='Images',
        on_delete=models.SET_NULL,
        help_text='Add pages and order them.',
        blank=True,
        null=True
    )
    just_descendants = models.BooleanField(
        'Select just descendants',
        help_text=_(
            'If checked and pages selected manually, only the descendants of the selected pages will be listed.'),
        default=False
    )
    page = PageField()

    class Meta:
        app_label = 'allink_cms'
        verbose_name = 'Page'
        verbose_name_plural = 'Pages'

    def __str__(self):
        return str(self.page.get_menu_title())


class AllinkLanguageChooserPlugin(CMSPlugin):
    cmsplugin_ptr = CMSPluginField()

    class Meta:
        app_label = 'allink_cms'



from unittest import mock
from django.test.testcases import TestCase
from django.template import Context, Template
from cms import api
from easy_thumbnails.exceptions import InvalidImageFormatError
from allink_core.core.test.base import PageApphookMixin, GenericPluginMixin
from allink_core.core.utils import get_ratio_w_h, get_height_from_ratio
from allink_core.core.test.factories import FilerImageFactory
from allink_core.core.templatetags.allink_image_tags import (
    get_sizes_from_width_alias, get_width_alias_from_plugin, render_image,
)
from allink_core.core_apps.allink_content.cms_plugins import CMSAllinkContentPlugin, CMSAllinkContentColumnPlugin
from allink_core.core_apps.allink_image.cms_plugins import CMSAllinkImagePlugin


class MockThumbnailer(mock.Mock):
    def get_thumbnail(*args, **kwargs):
        return None


THUMBNAIL_WIDTH_ALIASES = {
    '1-of-1': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 1200, 'ratio': '3-2'},
        'xl': {'width': 1500, 'ratio': '3-2'}
    },
    '1-of-2': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 650, 'ratio': '3-2'},
        'xl': {'width': 900, 'ratio': '3-2'}
    },
    '2-of-3': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 600, 'ratio': '3-2'},
        'xl': {'width': 800, 'ratio': '3-2'}
    },
    '1-of-3': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 400, 'ratio': '3-2'},
        'xl': {'width': 400, 'ratio': '3-2'}
    },
    '1-of-4': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 500, 'ratio': '3-2'},
        'xl': {'width': 500, 'ratio': '3-2'}
    },
    'test-original': {
        'xs': {'width': 450, 'ratio': 'x-y'},
        'sm': {'width': 500, 'ratio': 'x-y'},
        'xl': {'width': 500, 'ratio': 'x-y'}
    },
}


class ImageTagsUtilsTestCase(TestCase):

    def test_get_ratio_w_h(self):
        ratio = '3-2'
        expected_w_h = (3, 2)
        result = get_ratio_w_h(ratio)
        self.assertEqual(expected_w_h, result)

    def test_get_height_from_ratio(self):
        width = 450
        ratio_w = 3
        ratio_h = 2

        expected_h = width * ratio_h / ratio_w  # 300

        result = get_height_from_ratio(width, ratio_w, ratio_h)
        self.assertEqual(expected_h, result)

    def test_get_sizes_from_width_alias(self):
        alias = '1-of-1'
        expected_sizes = [
            ('xs', (450, 300)),
            ('sm', (1200, 800)),
            ('xl', (1500, 1000)),
        ]
        result = get_sizes_from_width_alias(alias, image=None)
        self.assertListEqual(expected_sizes, result)

    def test_get_sizes_from_width_alias_original(self):
        alias = 'test-original'
        expected_sizes = [
            ('xs', (450, 225)),
            ('sm', (500, 250)),
            ('xl', (500, 250)),
        ]

        image = mock.MagicMock()
        image.width = 1000
        image.height = 500

        result = get_sizes_from_width_alias(alias, image=image)
        self.assertListEqual(expected_sizes, result)


class ImageTagsImagePluginUtilsTestCase(GenericPluginMixin, TestCase):
    plugin_class = CMSAllinkImagePlugin

    def test_get_width_alias_from_plugin_no_context_3(self):
        result = get_width_alias_from_plugin(self.plugin_model_instance)
        self.assertEqual('2-of-3', result)


class ImageTagsContentPluginUtilsTestCase(PageApphookMixin, TestCase):
    template = 'col-1'
    position = 1  #

    def setUp(self):
        super().setUp()
        # AllinkContentPlugin, template='col-1'
        self.content_plugin = api.add_plugin(
            self.placeholder,
            CMSAllinkContentPlugin,
            self.language,
            template=self.template
        )
        # AllinkContentColumnPlugin, template='col-1'
        column_amount = CMSAllinkContentPlugin.model.get_template_column_count(self.content_plugin.template)

        for x in range(int(column_amount)):
            api.add_plugin(
                self.placeholder,
                CMSAllinkContentColumnPlugin,
                self.language,
                target=self.content_plugin
            )

        self.column_plugin = self.content_plugin.get_children()[self.position - 1]

        # AllinkImagePlugin, inside column plugin
        self.image_plugin = api.add_plugin(
            self.placeholder,
            CMSAllinkImagePlugin,
            self.language,
            target=self.column_plugin
        )

    def test_get_width_alias_from_plugin_no_context_1(self):
        """ template col-1 """

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('1-of-1', result)

    def test_get_width_alias_from_plugin_no_context_1_1(self):
        self.content_plugin.template = 'col-1-1'
        self.content_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('1-of-2', result)

    def test_get_width_alias_from_plugin_no_context_1_2_pos_0(self):
        self.content_plugin.template = 'col-1-2'
        self.content_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('1-of-3', result)

    def test_get_width_alias_from_plugin_no_context_1_2_pos_1(self):
        self.content_plugin.template = 'col-1-2'
        self.content_plugin.save()
        self.column_plugin.position = 1
        self.column_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('2-of-3', result)

    def test_get_width_alias_from_plugin_no_context_2_1_pos_0(self):
        self.content_plugin.template = 'col-2-1'
        self.content_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('2-of-3', result)

    def test_get_width_alias_from_plugin_no_context_2_1_pos_1(self):
        self.content_plugin.template = 'col-2-1'
        self.content_plugin.save()
        self.column_plugin.position = 1
        self.column_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('1-of-3', result)

    def test_get_width_alias_from_plugin_no_context_3(self):
        self.content_plugin.template = 'col-3'
        self.content_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('1-of-3', result)

    def test_get_width_alias_from_plugin_no_context_4(self):
        self.content_plugin.template = 'col-4'
        self.content_plugin.save()

        result = get_width_alias_from_plugin(self.image_plugin)
        self.assertEqual('1-of-4', result)


class ImageTagsImagePluginContextTestCase(TestCase):
    """
    used width_alias:

    '1-of-4': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 500, 'ratio': '3-2'},
        'xl': {'width': 500, 'ratio': '3-2'}
    },

    expected context:
    {
        'instance': {'id': 1},

        'image', Image:

        'icon_enabled': True,
        'bg_enabled': True,
        'bg_color': None,
        'lazyload_enabled': True,
        'alt_text': '',
        'vh_enabled': False,

        'ratio_percent_xs': '66.66666666666666%',
        'ratio_vh_xs': '66.66666666666666vh',
        'thumbnail_xs', ThumbnailFile: filer_public_thumbnails/filer_public/a6/34/a634ad24-214f-400c-8957-90c3a23d3da0/so.png__450x300.0_q60_HIGH_RESOLUTION_crop-smart_subsampling-2_upscale.jpg>,

        'ratio_percent_sm': '66.66666666666666%',
        'ratio_vh_sm': '66.66666666666666vh',
        'thumbnail_sm', ThumbnailFile: filer_public_thumbnails/filer_public/a6/34/a634ad24-214f-400c-8957-90c3a23d3da0/so.png__600x400.0_q60_HIGH_RESOLUTION_crop-smart_subsampling-2_upscale.jpg>,

        'ratio_percent_xl': '66.66666666666667%',
        'ratio_vh_xl': '66.66666666666667vh',
        'thumbnail_xl', ThumbnailFile: filer_public_thumbnails/filer_public/a6/34/a634ad24-214f-400c-8957-90c3a23d3da0/so.png__800x533.3333333333334_q60_HIGH_RESOLUTION_crop-smart_subsampling-2_upscale.jpg>
    }
    """

    width_alias = '1-of-4'

    def setUp(self):
        super().setUp()
        self.dummy_plugin = mock.Mock(id=1)

        self.context = {
            'instance': self.dummy_plugin
        }
        self.image = FilerImageFactory()

    def test_context_default_values(self):
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('instance'), self.dummy_plugin)
        self.assertEqual(context.get('icon_enabled'), False)
        self.assertEqual(context.get('bg_enabled'), True)
        self.assertEqual(context.get('bg_color'), None)
        self.assertEqual(context.get('lazyload_enabled'), True)
        self.assertEqual(context.get('vh_enabled'), False)

    def test_context_ratio_values_xs(self):
        # 'xs': {'width': 450, 'ratio': '3-2'},
        w = 450
        h = w / 3 * 2

        expected_ration = '{}'.format(h / w * 100)  # 66.66666666666666
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('ratio_percent_sm'), '{}%'.format(expected_ration))
        self.assertEqual(context.get('ratio_vh_sm'), '{}vh'.format(expected_ration))

    def test_context_ratio_values_sm(self):
        # 'sm': {'width': 500, 'ratio': '3-2'},
        w = 500
        h = w / 3 * 2

        expected_ration = '{}'.format(h / w * 100)  # 66.66666666666666
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('ratio_percent_xs'), '{}%'.format(expected_ration))
        self.assertEqual(context.get('ratio_vh_xs'), '{}vh'.format(expected_ration))

    def test_context_ratio_values_xl(self):
        # 'xl': {'width': 500, 'ratio': '3-2'}
        w = 500
        h = w / 3 * 2

        expected_ration = '{}'.format(h / w * 100)  # 66.66666666666667
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('ratio_percent_xl'), '{}%'.format(expected_ration))
        self.assertEqual(context.get('ratio_vh_xl'), '{}vh'.format(expected_ration))

    @mock.patch('allink_core.core.templatetags.allink_image_tags.get_thumbnailer', MockThumbnailer)
    def test_image_does_not_exist(self):
        """ if the image does not exist, no exception should not thrown """
        with mock.patch.object(MockThumbnailer, 'get_thumbnail') as mock_method:
            mock_method.side_effect = InvalidImageFormatError
            context = render_image(self.context, self.image, width_alias=self.width_alias)
            self.assertIsNone(context.get('thumbnail_xl'))

    @mock.patch('allink_core.core.templatetags.allink_image_tags.get_thumbnailer', MockThumbnailer)
    def test_thumbnail_options_default_values(self):
        # 'xl': {'width': 500, 'ratio': '3-2'}
        w = 500
        h = w / 3 * 2

        expected_thumbnail_options = {
            'crop': 'smart', 'bw': False, 'upscale': True, 'HIGH_RESOLUTION': True, 'zoom': None,
            'subject_location': self.image.subject_location, 'size': (w, h),
        }
        with mock.patch.object(MockThumbnailer, 'get_thumbnail') as mock_method:
            render_image(self.context, self.image, width_alias=self.width_alias)

            mock_method.assert_called_with(expected_thumbnail_options)
            self.assertEqual(mock_method.call_count, 3)


class ImageTagsImagePluginRenderTestCase(TestCase):
    """
    used width_alias:

    '1-of-4': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 500, 'ratio': '3-2'},
        'xl': {'width': 500, 'ratio': '3-2'}
    },
    """

    def setUp(self):
        super().setUp()
        self.dummy_plugin = mock.Mock(id=1)
        self.context = {
            'instance': self.dummy_plugin
        }
        self.image = FilerImageFactory()
        self.sizes = {
            'xs': "450x{}".format(450 / 3 * 2),
            'sm': "500x{}".format(500 / 3 * 2),
            'xl': "500x{}".format(500 / 3 * 2),
        }

    def test_context_render_all_image_urls(self):
        context = Context({
            'instance': self.dummy_plugin,
            'image': self.image,

        })
        template_to_render = Template(
            '{% load allink_image_tags %}'
            '{% render_image image width_alias="1-of-4" %}'
        )
        rendered_template = template_to_render.render(context)

        for size in self.sizes.items():
            self.assertIn('{}__{}'.format(self.image.original_filename, size[1]), rendered_template)



# # -*- coding: utf-8 -*-
from django.urls import path

from allink_core.core.loading import get_class

LocationsPluginLoadMore = get_class('locations.views', 'LocationsPluginLoadMore')
LocationsDetail = get_class('locations.views', 'LocationsDetail')
export_pdf = get_class('locations.views', 'export_pdf')


urlpatterns = [
    path('<int:page>/', LocationsPluginLoadMore.as_view(), name='more'),
    path('<slug:slug>/', LocationsDetail.as_view(), name='detail'),
    path('export-pdf/<int:id>/', export_pdf, name='export-pdf'),
]



# -*- coding: utf-8 -*-
from django.test.testcases import TestCase
from bs4 import BeautifulSoup
from allink_core.core.test import PageApphookMixin
from allink_core.apps.news.cms_apps import NewsApphook


class CMSHrefLangSitemapTestCase(PageApphookMixin, TestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'

    apphook_object = NewsApphook

    def test_status_code_200(self):
        response = self.client.get('/sitemap.xml')
        self.assertEqual(response.status_code, 200)

    def test_all_urls_with_hreflang_in_every_language(self):
        response = self.client.get('/sitemap.xml')
        xml = BeautifulSoup(response.content, 'xml').prettify()
        contains = [
            '<xhtml:link href="(.*?)page\/" hreflang="en" rel="alternate"\/>',
            '<xhtml:link href="(.*?)page-de\/" hreflang="de" rel="alternate"\/>',
            '<xhtml:link href="(.*?)page-fr\/" hreflang="fr" rel="alternate"\/>',
        ]
        for text in contains:
            self.assertRegex(xml, text)

    def test_all_urls_with_hreflang_in_every_language_child_page(self):
        response = self.client.get('/sitemap.xml')
        xml = BeautifulSoup(response.content, 'xml').prettify()
        contains = [
            '<xhtml:link href="(.*?)page\/child_page\/" hreflang="en" rel="alternate"\/>',
            '<xhtml:link href="(.*?)page-de\/child_page-de\/" hreflang="de" rel="alternate"\/>',
            '<xhtml:link href="(.*?)page-fr\/child_page-fr\/" hreflang="fr" rel="alternate"\/>',
        ]
        for text in contains:
            self.assertRegex(xml, text)

    def test_all_urls_with_hreflang_in_every_language_only_for_published_languages(self):
        self.child_page.unpublish('de')
        response = self.client.get('/sitemap.xml')
        xml = BeautifulSoup(response.content, 'xml').prettify()
        contains = [
            '<xhtml:link href="(.*?)page\/child_page\/" hreflang="en" rel="alternate"\/>',
            '<xhtml:link href="(.*?)page-fr\/child_page-fr\/" hreflang="fr" rel="alternate"\/>',
        ]
        for text in contains:
            self.assertRegex(xml, text)


# -*- coding: utf-8 -*-
from django.test.testcases import TestCase
from bs4 import BeautifulSoup
from allink_core.core.test import PageApphookMixin, DataModelMixin, DataModelTranslationMixin
from allink_core.apps.news.cms_apps import NewsApphook
from .factories import NewsFactory


class NewsSitemapsTestCase(PageApphookMixin, DataModelMixin, TestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'

    apphook_object = NewsApphook
    data_model_factory = NewsFactory

    def test_status_code_200(self):
        response = self.client.get('/sitemap.xml')
        self.assertEqual(response.status_code, 200)

    def test_all_urls_in_every_language(self):
        response = self.client.get('/sitemap.xml')
        xml = BeautifulSoup(response.content, 'xml').prettify()
        contains = [
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="de" rel="alternate"\/>'.format(self.entry_1.slug),
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="fr" rel="alternate"\/>'.format(self.entry_1.slug),
        ]
        for text in contains:
            self.assertNotRegex(xml, text)


class NewsSitemapsTranslatedTestCase(PageApphookMixin, DataModelTranslationMixin, TestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'

    apphook_object = NewsApphook
    data_model_factory = NewsFactory

    def test_all_urls_with_hreflang_in_every_language(self):
        response = self.client.get('/sitemap.xml')
        xml = BeautifulSoup(response.content, 'xml').prettify()
        contains = [
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="en" rel="alternate"\/>'.format(self.entry_1.slug),
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="de" rel="alternate"\/>'.format(self.entry_1.slug),
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="fr" rel="alternate"\/>'.format(self.entry_1.slug),
        ]
        for text in contains:
            self.assertRegex(xml, text)

    def test_all_urls_with_hreflang_in_every_language_only_for_published_languages(self):
        self.entry_1.delete_translation('de')
        response = self.client.get('/sitemap.xml')
        xml = BeautifulSoup(response.content, 'xml').prettify()
        contains = [
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="en" rel="alternate"\/>'.format(self.entry_1.slug),
            '<xhtml:link href="(.*?){0}(.*?)\/" hreflang="fr" rel="alternate"\/>'.format(self.entry_1.slug),
        ]
        for text in contains:
            self.assertRegex(xml, text)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django.db import models

from allink_core.core_apps.allink_button_link import fields


class LinkOrButton(models.CharField):
    default_field_class = fields.LinkOrButton
    south_field_class = 'django.db.models.fields.CharField'

    def __init__(self, *args, **kwargs):
        if 'verbose_name' not in kwargs:
            kwargs['verbose_name'] = 'Type'
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'blank' not in kwargs:
            kwargs['blank'] = False
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(LinkOrButton, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'choices_form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(LinkOrButton, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if '' in dict(self.choices).keys():
            kwargs['include_blank'] = False
        return super(LinkOrButton, self).get_choices(**kwargs)


class Context(models.CharField):
    default_field_class = fields.Context
    south_field_class = 'django.db.models.fields.CharField'

    def __init__(self, *args, **kwargs):
        if 'verbose_name' not in kwargs:
            kwargs['verbose_name'] = 'Context'
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'blank' not in kwargs:
            kwargs['blank'] = False
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(Context, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'choices_form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Context, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if '' in dict(self.choices).keys():
            kwargs['include_blank'] = False
        return super(Context, self).get_choices(**kwargs)


class Size(models.CharField):
    default_field_class = fields.Size
    south_field_class = 'django.db.models.fields.CharField'

    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = self.default_field_class.DEFAULT
        super(Size, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
            'choices_form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Size, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if '' in dict(self.choices).keys():
            kwargs['include_blank'] = False
        return super(Size, self).get_choices(**kwargs)


# TODO:
#   * btn-block, disabled
#   * pull-left, pull-right
#   * margins/padding



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib.postgres.fields import ArrayField
from django.utils.functional import cached_property
from django.utils.http import urlquote

from cms.models.pluginmodel import CMSPlugin
from filer.fields.image import FilerImageField
from filer.fields.file import FilerFileField

from allink_core.core.models.fields import Icon, CMSPluginField
from allink_core.core.models import AllinkLinkFieldsModel
from allink_core.core.models import choices

from allink_core.core_apps.allink_button_link import model_fields


class AllinkButtonLinkContainerPlugin(CMSPlugin):
    """
    A Container-Plugin for Links, Buttons
    """
    alignment_horizontal_desktop = models.CharField(
        'Alignment horizontal desktop',
        max_length=50,
        choices=choices.HORIZONTAL_ALIGNMENT_CHOICES,
        help_text='This option overrides the projects default alignment for desktop. (Usually "left")',
        blank=True,
        null=True
    )
    alignment_horizontal_mobile = models.CharField(
        'Alignment horizontal mobile',
        max_length=50,
        choices=choices.HORIZONTAL_ALIGNMENT_CHOICES,
        help_text='This option overrides the projects default alignment for mobile. (Usually "left")',
        blank=True,
        null=True
    )
    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )

    def __str__(self):
        return '{}'.format(str(self.pk))

    @property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        css_classes.append('align-h-desktop-{}'.format(
            self.alignment_horizontal_desktop)) if self.alignment_horizontal_desktop else None
        css_classes.append(
            'align-h-mobile-{}'.format(self.alignment_horizontal_mobile)) if self.alignment_horizontal_mobile else None
        return ' '.join(css_classes)


class AllinkButtonLinkPlugin(CMSPlugin, AllinkLinkFieldsModel):

    DEFAULT_LINK = 'default_link'
    FILE_LINK = 'file_link'
    IMAGE_LINK = 'image_link'
    PHONE_LINK = 'phone_link'
    EMAIL_LINK = 'email_link'
    FORM_LINK = 'form_link'
    VIDEO_EMBEDDED_LINK = 'video_embedded_link'
    VIDEO_FILE_LINK = 'video_file_link'

    TEMPLATE_CHOICES = (
        (DEFAULT_LINK, 'Internal/External'),
        (FILE_LINK, 'File (Download)'),
        (IMAGE_LINK, 'Image'),
        (PHONE_LINK, 'Phone'),
        (EMAIL_LINK, 'Email'),
        (FORM_LINK, 'Form'),
        (VIDEO_EMBEDDED_LINK, 'Video (Embedded)'),
        (VIDEO_FILE_LINK, 'Video (File)'),
    )

    # we re-use the template option to toggle fieldset visibility depending on the link types
    template = models.CharField(
        'Link type',
        help_text='Choose a link type in order to display its options below.',
        max_length=50,
        choices=TEMPLATE_CHOICES,
        default=DEFAULT_LINK
    )
    label = models.CharField(
        verbose_name='Link text',
        blank=True,
        default='',
        max_length=255,
    )
    type = model_fields.LinkOrButton(
        verbose_name='Display type',
    )
    # button specific fields
    btn_context = model_fields.Context(
        verbose_name='Variation',
    )
    btn_size = model_fields.Size(
        verbose_name='Size',
    )
    btn_block = models.BooleanField(
        verbose_name='Block',
        default=False,
    )
    # text link specific fields
    txt_context = model_fields.Context(
        verbose_name='Context',
        blank=True,
    )
    # common fields
    icon_left = Icon(
        verbose_name='Icon left',
    )
    icon_right = Icon(
        verbose_name='Icon right',
    )

    # email specific fields
    email_subject = models.CharField(
        verbose_name='Subject',
        max_length=255,
        default='',
        blank=True,
    )
    email_body_text = models.TextField(
        verbose_name='Body Text',
        default='',
        blank=True,
    )
    # form specific fields
    send_internal_mail = models.BooleanField(
        'Send internal e-mail',
        default=False,
        help_text='Send confirmation mail to defined internal e-mail addresses.'
    )
    internal_email_addresses = ArrayField(
        models.EmailField(
            blank=True,
            null=True,
        ),
        blank=True,
        null=True,
        verbose_name='Internal e-mail addresses',
    )
    from_email_address = models.EmailField(
        'Sender e-mail address',
        blank=True,
        null=True
    )
    send_external_mail = models.BooleanField(
        'Send external e-mail',
        default=False,
        help_text='Send confirmation mail to customer.'
    )
    thank_you_text = models.TextField(
        'Thank you text',
        blank=True,
        null=True,
        help_text='This text will be shown, after form completion.'
    )
    label_layout = models.CharField(
        'Display labels',
        max_length=15,
        choices=(
            ('stacked', 'Stacked with fields'),
            ('side_by_side', 'Side by side with fields'),
            ('placeholder', 'As placeholders'),
        ),
        default='stacked',
    )
    project_css_classes = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True
    )
    # video (embed) specific fields
    video_id = models.CharField(
        verbose_name='Video ID',
        max_length=255,
        help_text=(
           'Only provide the ID. The correct URL will automatically be generated.<br><br>'
           'YouTube: https://www.youtube.com/watch?v=<strong>12345678</strong> '
           '(the ID is <strong>12345678</strong>)<br>Vimeo: https://vimeo.com/<strong>12345678</strong> '
           '(the ID is <strong>12345678</strong>)'),
        blank=True,
        null=True,
    )
    video_service = models.CharField(
        'Video Service',
        max_length=50,
        choices=choices.VIDEO_SERVICE_CHOICES,
        blank=True,
        null=True,
    )
    ratio = models.CharField(
        'Ratio',
        max_length=50,
        blank=True,
        null=True
    )

    # video (file) specific fields
    video_file = FilerFileField(
        verbose_name='Video File',
        on_delete=models.PROTECT,
        help_text=(
           'Recommended video settings:<br><br>Format: mp4<br>Codec: H.264<br>Target Bitrate: 2 '
           '(video loads quick and runs smooth)<br>Audio: Not recommended (no audio = smaller file size and less '
           'annoyed visitors)<br>File size: Dependent of video length. Generally speaking: Less is more.'),
        blank=True,
        null=True,
        related_name='%(app_label)s_%(class)s_video_file',
    )
    video_poster_image = FilerImageField(
        verbose_name='Video Start Image',
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_video_poster_image',
        help_text=(
           'Image that is being displayed while the video is loading. Ideally the very first frame of the video '
           'is used, in order to make the transition as smooth as possible.<br><br><strong>Imoprtant:</strong> '
           'Make sure the aspect ratio of the image is <strong>exactly the same</strong> as the video, '
           'otherwise the video height will shrink or grow when the playback starts.'),
        blank=True,
        null=True,
    )
    video_muted_enabled = models.BooleanField(
        'Muted',
        help_text='Caution: Autoplaying videos with audio is not recommended. Use wisely.',
        default=True
    )
    # video (embed and file) specific fields
    auto_start_enabled = models.BooleanField(
        'Autostart',
        default=False,
        help_text='<strong>Important:</strong> Autoplaying videos with audio is not recommended. Use wisely. ',
    )
    allow_fullscreen_enabled = models.BooleanField(
        'Allow fullscreen',
        default=True
    )
    # modal closing options
    data_modal_escape_close_enabled = models.BooleanField(
        'Escape key closes modal',
        default=True,
    )
    data_modal_overlay_close_enabled = models.BooleanField(
        'Click on overlay closes modal',
        default=True,
    )
    data_modal_button_close_enabled = models.BooleanField(
        'Display close button',
        default=True,
    )

    cmsplugin_ptr = CMSPluginField()

    def __str__(self):
        return'{}'.format(self.label)

    @cached_property
    def link_url_typed(self):
        base = super(AllinkButtonLinkPlugin, self).link_url_typed
        link = base
        if self.link_mailto:
            parameters = {}
            if self.email_subject:
                parameters = {'subject': urlquote(self.email_subject)}
            if self.email_body_text:
                parameters['body'] = urlquote(self.email_body_text)
            link = '{}?{}'.format(base, '&'.join('{}={}'.format(v, k) for (v, k) in parameters.items()))
        return link

    @property
    def css_classes(self):
        css_classes = []
        if getattr(self, 'project_css_classes'):
            for css_class in getattr(self, 'project_css_classes'):
                css_classes.append(css_class)
        css_classes.append('side-by-side') if self.label_layout == 'side_by_side' else None
        css_classes.append('placeholder-enabled') if self.label_layout == 'placeholder' else None
        return ' '.join(css_classes)



# -*- coding: utf-8 -*-

from allink_core.core.loading import get_model
from allink_core.core.sitemap import HrefLangSitemap


News = get_model('news', 'News')


class NewsSitemap(HrefLangSitemap):

    changefreq = "never"
    priority = 0.5
    i18n = True

    def __init__(self, *args, **kwargs):
        self.namespace = kwargs.pop('namespace', None)
        super(NewsSitemap, self).__init__(*args, **kwargs)

    def items(self):
        return News.objects.translated()

    def lastmod(self, obj):
        return obj.modified



# -*- coding: utf-8 -*-
import datetime
from django.http import HttpResponse

from allink_core.core.views import AllinkBasePluginLoadMoreView, AllinkBaseDetailView
from allink_core.core.loading import get_model

from allink_core.apps.locations.pdf import PdfLocations


Locations = get_model('locations', 'Locations')
LocationsAppContentPlugin = get_model('locations', 'LocationsAppContentPlugin')


class LocationsPluginLoadMore(AllinkBasePluginLoadMoreView):
    model = Locations
    plugin_model = LocationsAppContentPlugin


class LocationsDetail(AllinkBaseDetailView):
    model = Locations


def export_pdf(request, id):
    date = (datetime.date.today().strftime('%d_%m_%Y'))

    item = Locations.objects.get(id=id)

    pdf = PdfLocations(item, request)
    output = pdf.build()
    filename = '%s_%s.pdf' % (item.title.replace(' ', '_'), date)

    response = HttpResponse(output.getvalue(), content_type="application/pdf")
    response['Content-Disposition'] = "attachment; filename=%s" % filename
    return response



# -*- coding: utf-8 -*-

from django.contrib import admin
from parler.admin import TranslatableAdmin
from webpack_loader.utils import get_files

from allink_core.core.admin.forms import AllinkCategoryAdminForm
from allink_core.core.admin.mixins import AllinkSEOAdminMixin, AllinkCategoryAdminMixin, AllinkMediaAdminMixin


class AllinkBaseAdmin(AllinkMediaAdminMixin, AllinkSEOAdminMixin, TranslatableAdmin):
    """
    Base ModelAdmin used in combination with AllinkBaseTranslatableModel
    """
    search_fields = ('translations__title',)
    list_display = ('title', 'status', 'created', 'modified', 'all_languages_column',)
    list_filter = ('status',)

    def get_fieldsets(self, request, obj=None):
        fieldsets = super().get_fieldsets()
        fieldsets += (
            (None, {
                'fields': (
                    'status',
                    'title',
                    'lead',
                    'preview_image',
                    'slug',
                    'created',
                ),
            }),
        )
        fieldsets += self.get_seo_fieldsets()
        return fieldsets


class AllinkCategoryAdmin(AllinkCategoryAdminMixin, AllinkBaseAdmin):
    """
    ModelAdmin used in combination with AllinkCategoryModel
    """
    list_display = ('title', 'status', 'all_categories_column', 'created', 'modified', 'all_languages_column',)
    list_filter = (
        'status',
        ('categories', admin.RelatedOnlyFieldListFilter,),
    )

    form = AllinkCategoryAdminForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'status',
                    'title',
                    'lead',
                    'preview_image',
                    'slug',
                    'created',
                )
            }),
        )
        fieldsets += self.get_seo_fieldsets()
        fieldsets += self.get_category_fieldsets()
        return fieldsets

    def all_categories_column(self, object):
        return "\n|\n".join([c.name for c in object.categories.all()])

    all_categories_column.short_description = 'Categories'



# -*- coding: utf-8 -*-
from django.conf import settings
from menus.menu_pool import menu_pool
from menus.base import Modifier
from cms.models import Page
from allink_core.core.loading import get_class


class AllinkPageMenuModifier(Modifier):

    def modify(self, request, nodes, namespace, root_id, post_cut, breadcrumb):
        # only do something when the menu has already been cut
        if post_cut:
            # only consider nodes that refer to cms pages
            # and put them in a dict for efficient access
            page_nodes = {n.id: n for n in nodes if n.attr["is_page"]}
            # retrieve the attributes of interest from the relevant pages
            pages = Page.objects.filter(id__in=page_nodes.keys(),
                                        allinkpageextension__isnull=False).values('id', 'allinkpageextension')
            # loop over all relevant pages
            for page in pages:
                # take the node referring to the page
                node = page_nodes[page['id']]
                # put the changed_by attribute on the node
                node.attr["special_nav"] = page['allinkpageextension']
        return nodes


if getattr(settings, 'ALLINK_PAGE_TOOLBAR_ENABLED', False):
    menu_pool.register_modifier(get_class('config.cms_menus', 'AllinkPageMenuModifier'))



from django.test.testcases import TestCase
from cms import api
from parler.utils.context import switch_language
from .factories import ConfigFactory, AllinkPageExtensionFactory, AllinkTitleExtensionFactory
from ..utils import get_meta_page_title, get_page_meta_dict, get_page_teaser_dict, get_fallback, get_meta_page_image_thumb, generate_meta_image_thumb


class AllinkExtensionMetaTestCase(TestCase):

    def setUp(self):
        self.page_de = api.create_page(
            title='page title',
            template='default.html',
            language='de',
            published=True,
        )
        self.allink_config = ConfigFactory()

    def test_get_page_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_meta_page_title(self.page_de), 'page title | default_base_title_en')

        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_meta_page_title(self.page_de), 'og title | default_base_title_en')

        self.page_de.get_title_obj().allinktitleextension.og_title = ''
        self.assertEqual(get_meta_page_title(self.page_de), 'teaser title | default_base_title_en')

        self.page_de.get_title_obj().allinktitleextension.teaser_title = ''
        self.assertEqual(get_meta_page_title(self.page_de), 'page title | default_base_title_en')

        # remove default_base_title_en from allink_config
        self.allink_config.default_base_title = ''
        # also remove default translation 'en'
        with switch_language(self.allink_config, 'en'):
            self.allink_config.default_base_title = ''
            self.allink_config.save()
        self.allink_config.save()
        self.assertEqual(get_meta_page_title(self.page_de), 'page title | ')

    def test_get_meta_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'meta_title'), 'page title')

        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_fallback(self.page_de, 'meta_title'), 'og title')

        self.page_de.get_title_obj().allinktitleextension.og_title = ''
        self.assertEqual(get_fallback(self.page_de, 'meta_title'), 'teaser title')

        self.page_de.get_title_obj().allinktitleextension.teaser_title = ''
        self.assertEqual(get_fallback(self.page_de, 'meta_title'), 'page title')

    def test_get_meta_description_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'meta_description'), None)

        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_fallback(self.page_de, 'meta_description'), 'og description')

        self.page_de.get_title_obj().allinktitleextension.og_description = ''
        self.assertEqual(get_fallback(self.page_de, 'meta_description'), 'teaser description')

        self.page_de.get_title_obj().allinktitleextension.teaser_description = None
        self.assertEqual(get_fallback(self.page_de, 'meta_description'), None)

    def test_get_meta_image_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'meta_image'), self.allink_config.default_og_image)

        # add page extensions
        AllinkPageExtensionFactory(extended_object=self.page_de)

        self.assertEqual(get_fallback(self.page_de, 'meta_image'), self.page_de.allinkpageextension.og_image)

        self.page_de.allinkpageextension.og_image = None
        self.assertEqual(get_fallback(self.page_de, 'meta_image'), self.page_de.allinkpageextension.teaser_image)

        self.page_de.allinkpageextension.teaser_image = None
        self.assertEqual(get_fallback(self.page_de, 'meta_image'), self.allink_config.default_og_image)

    def test_get_meta_dict(self):

        expected_meta_context = {
            'meta_page_title': get_meta_page_title(self.page_de),
            'meta_title': get_fallback(self.page_de, 'meta_title'),
            'meta_description': get_fallback(self.page_de, 'meta_description'),
            'meta_image_url': getattr(get_meta_page_image_thumb(self.page_de), 'url', ''),
            'google_site_verification': self.allink_config.google_site_verification,
        }
        self.assertDictEqual(expected_meta_context, get_page_meta_dict(self.page_de))
        self.assertIn('1200x630', str(get_page_meta_dict(self.page_de)))


class AllinkExtensionTeaserTestCase(TestCase):

    def setUp(self):
        self.page_de = api.create_page(
            title='page title',
            template='default.html',
            language='de',
            published=True,
        )
        self.allink_config = ConfigFactory()

    def test_get_teaser_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'teaser_title'), 'page title')

        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_fallback(self.page_de, 'teaser_title'), 'teaser title')

        self.page_de.get_title_obj().allinktitleextension.teaser_title = ''
        self.assertEqual(get_fallback(self.page_de, 'teaser_title'), 'page title')

    def test_get_teaser_description_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'teaser_description'), None)
        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_fallback(self.page_de, 'teaser_description'), 'teaser description')

        self.page_de.get_title_obj().allinktitleextension.teaser_description = ''
        self.assertEqual(get_fallback(self.page_de, 'teaser_description'), None)

    def test_get_teaser_technical_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'teaser_technical_title'), None)

        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_fallback(self.page_de, 'teaser_technical_title'), 'teaser technical title')

        self.page_de.get_title_obj().allinktitleextension.teaser_technical_title = ''
        self.assertEqual(get_fallback(self.page_de, 'teaser_technical_title'), None)

    def test_get_teaser_link_text_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'teaser_link_text'), 'Read more')

        # add page extensions
        AllinkTitleExtensionFactory(extended_object=self.page_de.get_title_obj())

        self.assertEqual(get_fallback(self.page_de, 'teaser_link_text'), 'teaser link text')

    def test_get_teaser_image_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.page_de, 'teaser_image'), None)

        # add page extensions
        AllinkPageExtensionFactory(extended_object=self.page_de)

        self.assertEqual(get_fallback(self.page_de, 'teaser_image'), self.page_de.allinkpageextension.teaser_image)

        self.page_de.allinkpageextension.teaser_image = None
        self.assertEqual(get_fallback(self.page_de, 'teaser_image'), None)

    def test_get_teaser_dict(self):
        AllinkPageExtensionFactory(extended_object=self.page_de)

        expected_meta_context = {
            'teaser_title': get_fallback(self.page_de, 'teaser_title'),
            'teaser_technical_title': get_fallback(self.page_de, 'teaser_technical_title'),
            'teaser_description': get_fallback(self.page_de, 'teaser_description'),
            'teaser_image': get_fallback(self.page_de, 'teaser_image'),
            'teaser_link_text': get_fallback(self.page_de, 'teaser_link_text'),
            'teaser_link': self.page_de.get_absolute_url(),
        }
        self.assertDictEqual(expected_meta_context, get_page_teaser_dict(self.page_de))


class AllinkUtilsTestCase(TestCase):

    def setUp(self):
        self.page_de = api.create_page(
            title='page title',
            template='default.html',
            language='de',
            published=True,
        )
        self.allink_config = ConfigFactory()

    # def test_get_fallback_invalid_key(self):
    #     get_fallback()

    def test_generate_meta_image_thumb_correct_size(self):
        meta_image = get_fallback(self.page_de, 'meta_image')
        thumb = generate_meta_image_thumb(meta_image)

        self.assertEqual(thumb.image.width, 1200)
        self.assertEqual(thumb.image.height, 630)

    def test_generate_meta_image_thumb_none(self):
        meta_image = None
        thumb = generate_meta_image_thumb(meta_image)
        self.assertIsNone(thumb)



# -*- coding: utf-8 -*-
from allink_core.apps.locations.abstract_models import BaseLocations, BaseLocationsTranslation, \
    BaseLocationsAppContentPlugin
from allink_core.core.loading import get_model
from allink_core.core.loading import is_model_registered

__all__ = []


if not is_model_registered('locations', 'Locations'):
    class Locations(BaseLocations):
        pass

    __all__.append('Locations')


if not is_model_registered('locations', 'LocationsTranslation'):
    class LocationsTranslation(BaseLocationsTranslation):
        pass

    __all__.append('LocationsTranslation')


if not is_model_registered('locations', 'LocationsAppContentPlugin'):
    class LocationsAppContentPlugin(BaseLocationsAppContentPlugin):
        data_model = get_model('locations', 'Locations')

    __all__.append('LocationsAppContentPlugin')



# Generated by Django 2.1.10 on 2019-10-02 20:20

import allink_core.core.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0004_remove_newsappcontentplugin_filter_fields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='newsappcontentplugin',
            name='manual_entries',
            field=allink_core.core.models.fields.SortedM2MModelField(blank=True, help_text='Select and arrange specific entries, or, leave blank to select all. (If manual entries are selected the category filtering will be applied as well.)', to='news.News'),
        ),
    ]



# Generated by Django 2.1.8 on 2019-07-02 17:18

import adminsortable.fields
import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkLanguageChooserPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_cms_allinklanguagechooserplugin', serialize=False, to='cms.CMSPlugin')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AllinkPage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('just_descendants', models.BooleanField(default=False, help_text='If checked and pages selected manually, only the descendants of the selected pages will be listed.', verbose_name='Select just descendants')),
                ('page', cms.models.fields.PageField(on_delete=django.db.models.deletion.CASCADE, to='cms.Page')),
            ],
            options={
                'verbose_name': 'Page',
                'verbose_name_plural': 'Pages',
            },
        ),
        migrations.CreateModel(
            name='AllinkPageChooserPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_cms_allinkpagechooserplugin', serialize=False, to='cms.CMSPlugin')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.AddField(
            model_name='allinkpage',
            name='pagechooser',
            field=adminsortable.fields.SortableForeignKey(blank=True, help_text='Add pages and order them.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='allink_cms.AllinkPageChooserPlugin', verbose_name='Images'),
        ),
    ]



# -*- coding: utf-8 -*-
from django import forms
from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase

from allink_core.core_apps.allink_info_box.models import AllinkInfoBoxPlugin


class AllinkInfoBoxPluginForm(forms.ModelForm):

    class Meta:
        model = AllinkInfoBoxPlugin
        fields = (
            'template',
            'counter',
            'transparent_background'
        )

    def __init__(self, *args, **kwargs):
        super(AllinkInfoBoxPluginForm, self).__init__(*args, **kwargs)
        self.fields['template'] = forms.CharField(
            label='Template',
            widget=forms.Select(choices=self.instance.get_templates()),
            required=True,
        )


@plugin_pool.register_plugin
class CMSAllinkInfoBoxPlugin(CMSPluginBase):
    model = AllinkInfoBoxPlugin
    name = 'Info Box'
    module = 'Generic'
    cache = False
    allow_children = True
    child_classes = ['CMSAllinkContentPlugin']
    form = AllinkInfoBoxPluginForm

    def get_render_template(self, context, instance, placeholder, file='content'):
        template = 'allink_info_box/plugins/{}/{}.html'.format(instance.template, file)
        return template



# Generated by Django 2.2.6 on 2019-10-25 14:10

import allink_core.core_apps.allink_button_link.model_fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('allink_button_link', '0002_auto_20191002_1951'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinkbuttonlinkplugin',
            name='btn_context',
            field=allink_core.core_apps.allink_button_link.model_fields.Context(default='default', max_length=255, verbose_name='Variation'),
        ),
        migrations.AlterField(
            model_name='allinkbuttonlinkplugin',
            name='txt_context',
            field=allink_core.core_apps.allink_button_link.model_fields.Context(blank=True, default='default', max_length=255, verbose_name='Context'),
        ),
    ]



# -*- coding: utf-8 -*-
import json

from django import forms

from cms.models import Page

from allink_core.core.forms.widgets import SearchSelectWidget


class AllinkInternalLinkFieldMixin(forms.ModelForm):
    # additional = SelectLinkField()

    def __init__(self, *args, **kwargs):
        super(AllinkInternalLinkFieldMixin, self).__init__(*args, **kwargs)
        for field_name, field in filter(lambda x: isinstance(x[1].widget, SearchSelectWidget), self.fields.items()):
            self.fields[field_name].choices = self.fields[field_name].get_page_and_app_choices()
            if self.instance:
                if self.instance.link_page:
                    self.fields[field_name].initial = json.dumps({'page_id': self.instance.link_page.id})
                elif self.instance.link_apphook_page:
                    self.fields[field_name].initial = json.dumps({
                        'link_apphook_page_id': self.instance.link_apphook_page.id,
                        'link_url_name': self.instance.link_url_name,
                        'link_object_id': self.instance.link_object_id,
                        'link_url_kwargs': self.instance.link_url_kwargs,
                        'link_model': self.instance.link_model
                    })

    def clean(self):
        cleaned_data = super(AllinkInternalLinkFieldMixin, self).clean()
        for field_name, field in filter(lambda x: isinstance(x[1].widget, SearchSelectWidget), self.fields.items()):
            self.clean_search_select_field(field_name, field, cleaned_data)
        return cleaned_data

    def clean_search_select_field(self, field_name, field, cleaned_data):
        if cleaned_data[field_name]:
            field_data = json.loads(self.cleaned_data[field_name])
            if 'page_id' in field_data:
                self.instance.link_page = Page.objects.get(id=field_data['page_id'])
                self.instance.link_apphook_page = None
                self.instance.link_object_id = None
                self.instance.link_model = None
                self.instance.link_url_name = None
                self.instance.link_url_kwargs = None
            elif 'link_apphook_page_id' in field_data:
                self.instance.link_page = None
                self.instance.link_apphook_page = Page.objects.get(id=field_data['link_apphook_page_id'])
                self.instance.link_object_id = field_data['link_object_id']
                self.instance.link_model = field_data['link_model']
                self.instance.link_url_name = field_data['link_url_name']
                self.instance.link_url_kwargs = field_data['link_url_kwargs']
        else:
            self.instance.link_page = None
            self.instance.link_apphook_page = None
            self.instance.link_object_id = None
            self.instance.link_model = None
            self.instance.link_url_name = None
            self.instance.link_url_kwargs = None



# -*- coding: utf-8 -*-
from menus.base import NavigationNode
from menus.menu_pool import menu_pool
from cms.menu_bases import CMSAttachMenu

from allink_core.core.loading import get_model, get_class


Locations = get_model('locations', 'Locations')


class LocationsMenu(CMSAttachMenu):

    name = "Locations menu"

    def get_nodes(self, request):

        nodes = []
        for entry in Locations.objects.active():
            node = NavigationNode(
                entry.title,
                entry.get_absolute_url(),
                entry.sort_order,
                attr={
                    'description': entry.lead
                })
            nodes.append(node)

        return nodes


menu_pool.register_menu(get_class('locations.cms_menus', 'LocationsMenu'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django import forms


class Context(forms.widgets.RadioSelect):
    template_name = 'admin/allink_button_link/widgets/context.html'


class Size(forms.widgets.RadioSelect):
    template_name = 'admin/allink_button_link/widgets/size.html'


class MiniTextarea(forms.widgets.Textarea):
    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}
        attrs['cols'] = '120'
        attrs['rows'] = '1'
        super(MiniTextarea, self).__init__(attrs)


class LinkOrButtonRenderer(forms.widgets.RadioSelect):
    def render(self):
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/allink_button_link/widgets/link_or_button.html',
            {'selects': self},
        )
        return rendered


class LinkOrButton(forms.widgets.RadioSelect):
    renderer = LinkOrButtonRenderer



# -*- coding: utf-8 -*-
import requests
from io import BytesIO
from requests.exceptions import MissingSchema
from PIL import Image as PILImage

from django.utils.translation import activate

from reportlab.platypus import BaseDocTemplate
from reportlab.lib.pagesizes import A4
from reportlab.platypus import (Paragraph, PageTemplate, Frame, NextPageTemplate, Image, FrameBreak, PageBreak)
from reportlab.lib.styles import ParagraphStyle, StyleSheet1
from reportlab.lib.colors import Color
from reportlab.platypus.flowables import KeepTogether, ListFlowable, ListItem, HRFlowable
from reportlab.lib.utils import ImageReader, simpleSplit
from reportlab.lib.units import mm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfgen import canvas
from reportlab.lib.colors import HexColor
from bs4 import BeautifulSoup

from cms.utils.plugins import get_plugins
from allink_core.core.utils import get_height_from_ratio, get_ratio_w_h


def clean(value):
    return value.replace('&', '&amp;')


pt = 0.353


def has_class(tag, cls):
    try:
        return True if tag['class'][0] == cls else False
    except KeyError:
        return False


def read_file_open(image):
    """
    used for drawImage
    reads a file from boto with its url/ and opens it
    returns a ImageReader object which can be directly be drawn
    needed, beacause media files cant be loaded directly from file stystem
    (they are stored on s3)
    """
    try:
        return PILImage.open(BytesIO(requests.get(image.url).content))
    except MissingSchema:
        # localhost (read file from local file system)
        return image.path


def read_file(image):
    """
    used for Image(Flowable)
    reads a file from boto with its url
    returns a ImageReader object which can be directly be drawn
    needed, beacause media files cant be loaded directly from file stystem
    (they are stored on s3)
    """
    try:
        return BytesIO(requests.get(image.url).content)
    except MissingSchema:
        # localhost (read file from local file system)
        return image.path


def set_links(tag):
    import re

    from cms.models.pluginmodel import CMSPlugin
    from allink_core.core.utils import base_url

    cleaned = str(tag)
    links = tag.findAll('cms-plugin')
    for link in links:
        if link['alt'].startswith('Button'):
            button = CMSPlugin.objects.get(id=link['id']).get_plugin_instance()[0]
            btn_reg = re.compile('<cms-plugin alt="Button.*?id="%s".*?</cms-plugin>' % link['id'])
            cleaned = re.sub(btn_reg.pattern, r'<link href="%s"><u>%s</u></link>'
                             % (base_url() + button.link_url_typed,
                                button.label), str(cleaned),
                             flags=re.DOTALL)
    return cleaned


def extract_content_from_text_plugin(plugin):
    """
    returns a list with tuples
    ('title-h1', '<<content>>'),
    ...
    """
    body = plugin.get_plugin_instance()[0].body
    soup = BeautifulSoup(body)
    tags = soup.find_all(['p', 'h1', 'h2', 'h3', 'ul'])
    content = ()
    for tag in tags:
        # title
        if tag.name == 'h1':
            tag = set_links(tag)
            content += ('title-h1', str(tag)),
        elif tag.name == 'h2':
            tag = set_links(tag)
            content += ('title-h2', str(tag)),
        elif tag.name == 'h3':
            tag = set_links(tag)
            content += ('title-h3', str(tag)),
        # text
        elif has_class(tag, 'lead'):
            tag = set_links(tag)
            content += ('text-lead', str(tag)),
        elif has_class(tag, 'medium'):
            tag = set_links(tag)
            content += ('text-medium', str(tag)),
        elif has_class(tag, 'small'):
            tag = set_links(tag)
            content += ('text-small', str(tag)),
        # lists
        elif tag.name == 'ul':
            content += ('list-bullet', map(lambda x: set_links(x), tag.findAll('li'))),
        else:
            # fallback and if no class is defined
            tag = set_links(tag)
            content += ('text-normal', str(tag)),
    return content


class NumberedCanvas(canvas.Canvas):
    def __init__(self, *args, **kwargs):
        canvas.Canvas.__init__(self, *args, **kwargs)
        self._saved_page_states = []

    def showPage(self):
        self._saved_page_states.append(dict(self.__dict__))
        self._startPage()

    def save(self):
        """add page info to each page (page x of y)"""
        num_pages = len(self._saved_page_states)
        for state in self._saved_page_states:
            self.__dict__.update(state)
            self.draw_page_number(num_pages)
            canvas.Canvas.showPage(self)
        canvas.Canvas.save(self)

    def draw_page_number(self, page_count):
        if self._pageNumber > 1:
            self.setFont("MessinaSansRegular", 7.5, 9.5)
            self.drawRightString(147.25 * mm, 288 * mm, '{}/{}'.format(self._pageNumber, page_count))


class PdfLocations:

    page_templates = None

    floatings = []

    pagesize = A4
    rightMargin = 6.5 * mm
    leftMargin = 18 * mm
    topMargin = 7 * mm
    bottomMargin = 7 * mm

    def __init__(self, item, request, *args, **kwargs):

        pdfmetrics.registerFont(TTFont('AerotypeZoojaLight', 'static/fonts/pdf/Aerotype-Zooja-Light.ttf'))
        pdfmetrics.registerFont(TTFont('MessinaSansBold', 'static/fonts/pdf/MessinaSans-Bold.ttf'))
        pdfmetrics.registerFont(TTFont('MessinaSansRegular', 'static/fonts/pdf/MessinaSans-Regular.ttf'))

        pdfmetrics.registerFontFamily('MessinaSans', normal='MessinaSansRegular', bold='MessinaSansBold',)
        pdfmetrics.registerFontFamily('AerotypeZoojaLight', normal='AerotypeZoojaLight',)

        self.output = BytesIO()
        self.item = item
        self.language = request.LANGUAGE_CODE
        self.logo = ImageReader('static/images/branding/hr-campus-logo-and-claim.png')

        self.doc = BaseDocTemplate(
            filename=self.output,
            pagesize=self.pagesize,
            rightMargin=self.rightMargin,
            leftMargin=self.leftMargin,
            topMargin=self.topMargin,
            bottomMargin=self.bottomMargin
        )
        self.first_page_settings = {
            'header_height': 5 * mm,
            'footer_height': 28 * mm,
            'preview_image_width': 52.75 * mm,
            'preview_image_height': 52.75 * mm,
        }
        self.later_page_settings = {
            'header_height': 15 * mm,
            'footer_height': 28 * mm,
        }
        self.frames = {}

        self.request = request
        self.transparent = Color(0, 0, 0, alpha=0.7)
        self.brand_color = HexColor(0xFAE600)
        self.font_color = HexColor(0x43494d)
        self.init_page_templates()
        self.styles = self.get_stylesheets()

        # contnet from placeholders
        self.allowed_header_plugins = [
            'TextPlugin',
        ]
        self.allowed_content_plugins = [
            'TextPlugin',
            'CMSAllinkImagePlugin',
            'CMSAllinkPageBreakPlugin',
        ]

        self.header_plugins = self.get_relevant_header_plugins()
        self.content_plugins = self.get_relevant_content_plugins()

    def init_page_templates(self):

        self.doc.addPageTemplates(
            [
                PageTemplate(
                    id='first_page',
                    pagesize=self.pagesize,
                    onPage=self.on_first_page,
                    frames=[
                        Frame(
                            152.25 * mm,
                            0,
                            width=self.first_page_settings['preview_image_width'],
                            height=233.75 * mm,
                            bottomPadding=(2 * self.bottomMargin) + self.first_page_settings['footer_height'],
                            leftPadding=0,
                            rightPadding=0,
                        ),
                        Frame(
                            self.leftMargin,
                            0,
                            width=129.25 * mm,
                            height=self.doc.height,
                            bottomPadding=self.first_page_settings['footer_height'],
                            topPadding=0,
                            leftPadding=0,
                            rightPadding=0,
                        )
                    ]
                ),
                PageTemplate(
                    id='later_page',
                    pagesize=A4,
                    onPage=self._later_pages,
                    frames=[
                        Frame(
                            self.leftMargin,
                            0,
                            width=129.25 * mm,
                            height=self.doc.height,
                            bottomPadding=self.later_page_settings['footer_height'],
                            topPadding=0,
                            leftPadding=0,
                            rightPadding=0,
                        )
                    ]
                )
            ]
        )

        self.floatings.append(NextPageTemplate('later_page'))

    def get_stylesheets(self):
        stylesheet = StyleSheet1()

        # PDF styles
        stylesheet.add(ParagraphStyle(name='pre-title',
                                      fontName='MessinaSansBold',
                                      fontSize=9,
                                      textColor=self.font_color,
                                      leading=9.5
                                      )
                       )
        stylesheet.add(ParagraphStyle(name='main-title',
                                      fontName='AerotypeZoojaLight',
                                      fontSize=48,
                                      textColor=self.font_color,
                                      leading=45,
                                      spaceAfter=7 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='sub-title',
                                      fontName='MessinaSansBold',
                                      fontSize=14,
                                      textColor=self.font_color,
                                      leading=22,
                                      spaceAfter=17.25 * mm)
                       )
        # ckeditor styles
        stylesheet.add(ParagraphStyle(name='title-h1',
                                      fontName='MessinaSansBold',
                                      fontSize=9,
                                      textColor=self.font_color,
                                      leading=9.5,
                                      spaceAfter=2.25 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='title-h2',
                                      fontName='AerotypeZoojaLight',
                                      fontSize=48,
                                      textColor=self.font_color,
                                      leading=45,
                                      spaceAfter=7 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='title-h3',
                                      fontName='MessinaSansBold',
                                      fontSize=14,
                                      textColor=self.font_color,
                                      leading=22,
                                      spaceAfter=0.2 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='text-lead',
                                      fontName='MessinaSansBold',
                                      fontSize=14,
                                      textColor=self.font_color,
                                      leading=22,
                                      spaceAfter=7 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='text-medium',
                                      fontName='MessinaSansRegular',
                                      fontSize=14,
                                      textColor=self.font_color,
                                      leading=18,
                                      spaceAfter=7 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='text-normal',
                                      fontName='MessinaSansRegular',
                                      fontSize=10,
                                      textColor=self.font_color,
                                      leading=14,
                                      spaceAfter=5.25 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='text-normal-link',
                                      fontName='MessinaSansRegular',
                                      fontSize=10,
                                      textColor=self.font_color,
                                      leading=14,
                                      spaceAfter=5.25 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='text-small',
                                      fontName='MessinaSansRegular',
                                      fontSize=8.5,
                                      textColor=self.font_color,
                                      leading=14,
                                      spaceAfter=3.25 * mm)
                       )
        stylesheet.add(ParagraphStyle(name='list-bullet',
                                      fontName='MessinaSansRegular',
                                      fontSize=10,
                                      textColor=self.font_color,
                                      leading=14,)
                       )
        stylesheet.add(ParagraphStyle(name='list-number',
                                      fontName='MessinaSansRegular',
                                      fontSize=10,
                                      textColor=self.font_color,
                                      leading=14,
                                      spaceAfter=5.25 * mm)
                       )
        return stylesheet

    def draw_main_image(self, canvas, doc):
        canvas.setFillColor(self.brand_color)
        canvas.rect(152.25 * mm, self.doc.height - 43.75 * mm, 52.75 * mm, 52.75 * mm, stroke=0, fill=1)
        if self.item.preview_image:
            canvas.drawImage(ImageReader(read_file_open(self.item.preview_image)), 152.25 * mm,
                             self.doc.height - 43.75 * mm, mask='auto', width=52.75 * mm, height=52.75 * mm)

    def draw_footer(self, canvas, doc):
        canvas.setFont('MessinaSansRegular', 7.5, 9.5)
        canvas.setFillColor(self.font_color)
        w, h = self.logo.getSize()
        ratio = h / float(w)
        # static files can be loaded directly from file stystem
        canvas.drawImage(self.logo, self.leftMargin, 7 * mm, mask='auto', width=49.7 * mm, height=(49.7 * mm * ratio))

        canvas.drawString(107.5 * mm, 22 * mm - self.bottomMargin, 'HR Campus AG')
        canvas.drawString(107.5 * mm, 18.5 * mm - self.bottomMargin, 'Kriesbachstrasse 3')
        canvas.drawString(107.5 * mm, 15 * mm - self.bottomMargin, 'CH-8600 Dübendorf')

        canvas.drawString(152.25 * mm, 22 * mm - self.bottomMargin, '+41 44 215 15 15')
        canvas.drawString(152.25 * mm, 18.5 * mm - self.bottomMargin, 'office@hr-campus.ch')
        canvas.drawString(152.25 * mm, 15 * mm - self.bottomMargin, 'www.hr-campus.ch')

    def draw_line(self, canvas, doc):
        canvas.line(doc.leftMargin, 22 * mm, 52.75 * mm + 129.25 * mm, 22 * mm)

    def on_first_page(self, canvas, doc):
        self.draw_main_image(canvas, doc)
        self.draw_line(canvas, doc)
        self.draw_footer(canvas, doc)

    def _later_pages(self, canvas, doc):
        canvas.setFont('MessinaSansBold', 7.5, 9.5)
        # break line when title to long
        title_area = simpleSplit(clean(self.item.title), canvas._fontname, canvas._fontsize,
                                 self.first_page_settings['preview_image_width'])
        x = 152.25 * mm
        y = 288 * mm
        for line in title_area:
            canvas.drawString(x, y, line)
            y -= canvas._leading

        # canvas.drawString(152.25 * mm, 288 * mm, clean(self.item.title))
        self.draw_line(canvas, doc)
        self.draw_footer(canvas, doc)

    def get_relevant_header_plugins(self):
        """
        returns a list with only relevant plugins which are allowed in header_placeholder
        Plugins are directly inside the placeholder
        """
        all_plugins = get_plugins(
            request=self.request,
            placeholder=self.item.header_placeholder,
            template=None,
        )
        relevant_plugins = []

        for plugin in all_plugins:
            if plugin.plugin_type in self.allowed_header_plugins:
                relevant_plugins.append(plugin)
            elif plugin.plugin_type == 'CMSAllinkContentPlugin' and not plugin.ignore_in_pdf:
                relevant_plugins.append(plugin)
        return relevant_plugins

    def get_relevant_content_plugins(self):
        """
        returns a list with only relevant plugins from a queryset of plugins
        Plugins are always children of CMSAllinkContentColumnPlugin or CMSAllinkContentPlugin itself
        """
        all_plugins = get_plugins(
            request=self.request,
            placeholder=self.item.content_placeholder,
            template=None,
            lang=self.language,
        )
        relevant_plugins = []
        for plugin in all_plugins:
            if plugin.plugin_type == 'CMSAllinkContentPlugin':
                # at the moment only templates 'col-1' and 'col-1-1' are supported
                if not plugin.ignore_in_pdf and (plugin.template == 'col-1' or plugin.template == 'col-1-1'):
                    relevant_plugins.append(plugin)
                    column_plugins = plugin.get_children()
                    for column in column_plugins:
                        for child in column.get_children():
                            # exlude images which are used as background icons
                            if not getattr(child.get_plugin_instance()[0], 'project_css_classes', False):
                                # also skip ImagePlugins which are inside a content plugin with template col-1-1
                                if plugin.template == 'col-1-1' and child.plugin_type == 'CMSAllinkImagePlugin':
                                    continue
                                if child.plugin_type in self.allowed_content_plugins:
                                    relevant_plugins.append(child.get_plugin_instance()[0])
            elif plugin.plugin_type == 'CMSAllinkPageBreakPlugin':
                relevant_plugins.append(plugin)
        return relevant_plugins

    def append_pagebreak_plugin(self, plugin):
        self.floatings.append(PageBreak())

    def append_text_plugin(self, plugin):
        content = extract_content_from_text_plugin(plugin)

        for tag in content:
            # all types of lists
            if tag[0] == 'list-bullet' or tag[0] == 'list-number':
                list_flowables = []
                for item in tag[1]:
                    list_flowables.append(ListItem(Paragraph(item, self.styles[tag[0]]),
                                                   leftIndent=10, value='circle'))
                self.floatings.append(ListFlowable(list_flowables,
                                                   bulletType='bullet',
                                                   start='circle',
                                                   leftIndent=10,
                                                   bulletFontSize=3,
                                                   bulletOffsetY=-4,
                                                   spaceAfter=4 * mm))
            # all types paragraph styles
            # title - h1, title - h2, title - h3,
            # text - lead, text - normal, text - small
            else:
                self.floatings.append(KeepTogether(Paragraph(tag[1], self.styles[tag[0]])))
                if tag[0] == 'title-h3':
                    self.floatings.append(HRFlowable(spaceBefore=0.2 * mm, spaceAfter=5.25 * mm,
                                                     color=self.transparent, thickness=0.5, width='100%'))

    def append_image_plugin(self, plugin):
        if hasattr(plugin.djangocms_image_allinkimageplugin, 'picture'):
            picture_plug = plugin.djangocms_image_allinkimageplugin
            if picture_plug.picture:
                image = picture_plug.picture
                width = 129.25 * mm  # is same as Frame.width
                # use ratio from plugin
                if picture_plug.ratio and picture_plug.ratio != 'x-y':
                    ratio_w, ratio_h = get_ratio_w_h(picture_plug.ratio)
                # use original ratio
                else:
                    ratio_w = image.width
                    ratio_h = image.height
                height = get_height_from_ratio(width, ratio_w, ratio_h)

                self.floatings.append(Image(read_file(image), width=width, height=height,
                                            mask='auto', hAlign='LEFT', ))
                self.floatings.append(Paragraph('', self.styles['main-title']))

    def append_content_plugin(self, plugin):
        # render the title of the section in the corresponding style
        # styles: title - h1, title - h2, title - h3

        if plugin.title:
            self.floatings.append(KeepTogether(Paragraph(clean(plugin.title),
                                                         self.styles['title-{}'.format(plugin.title_size)])))
            if plugin.title_size == 'h3':
                self.floatings.append(HRFlowable(spaceBefore=0.2 * mm, spaceAfter=5.25 * mm,
                                                 color=self.transparent, thickness=0.5, width='100%'))

    def append_plugins(self, plugins):
        for plugin in plugins:
            if plugin.plugin_type == 'CMSAllinkPageBreakPlugin':
                self.append_pagebreak_plugin(plugin)
            elif plugin.plugin_type == 'TextPlugin':
                self.append_text_plugin(plugin)
            elif plugin.plugin_type == 'CMSAllinkImagePlugin':
                self.append_image_plugin(plugin)
            elif plugin.plugin_type == 'CMSAllinkContentPlugin':
                self.append_content_plugin(plugin)

    def build(self):
        activate(self.language)

        self.floatings.append(HRFlowable(spaceBefore=0.2 * mm, spaceAfter=5.25 * mm, color=self.transparent,
                                         thickness=0.5, width='100%'))

        self.floatings.append(FrameBreak())

        # page title
        self.floatings.append(Paragraph(clean(self.request.current_page.get_title()), self.styles['pre-title']))

        # main title
        self.floatings.append(Paragraph(clean(self.item.title), self.styles['main-title']))

        # sub title
        self.floatings.append(Paragraph(clean(self.item.lead), self.styles['sub-title']))

        # content
        self.append_plugins(self.header_plugins)
        self.append_plugins(self.content_plugins)

        self.doc.build(self.floatings, canvasmaker=NumberedCanvas)
        return self.output



# -*- coding: utf-8 -*-
from django.urls import path
from allink_core.core.loading import get_class

NewsPluginLoadMore = get_class('news.views', 'NewsPluginLoadMore')
NewsDetail = get_class('news.views', 'NewsDetail')

urlpatterns = [
    path('<int:page>/', NewsPluginLoadMore.as_view(), name='more'),
    path('<slug:slug>/', NewsDetail.as_view(), name='detail'),
]



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse

from cms.toolbar.items import Break
from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER, ADMIN_SITES_BREAK
from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from allink_core.core_apps.allink_legacy_redirect.models import AllinkLegacyLink


@toolbar_pool.register
class AllinkLegacyRedirectToolbar(CMSToolbar):
    model = AllinkLegacyLink

    def populate(self):

        admin_menu = self.toolbar.get_menu(ADMIN_MENU_IDENTIFIER)
        position = admin_menu.find_first(Break, identifier=ADMIN_SITES_BREAK)
        allink_menu = admin_menu.get_or_create_menu(
            'allink-menu',
            'allink',
            position=position
        )

        url = reverse('admin:{}_{}_changelist'.format(self.model._meta.app_label, self.model._meta.model_name))
        allink_menu.add_sideframe_item('Legacy Redirect', url=url)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.db import models

from cms.models.fields import PageField
from cms.models.fields import PlaceholderField
from adminsortable.models import SortableMixin
from parler.models import TranslatedField
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.image import FilerImageField

from allink_core.core.models.fields import SortedM2MModelField
from allink_core.core.models import (
    AllinkCategoryFieldsModel,
    AllinkBaseTranslatableModel,
    AllinkBaseAppContentPlugin,
    AllinkBaseTranslatedFieldsModel,
)

from .managers import DummyAppManager


class DummyApp(SortableMixin, AllinkCategoryFieldsModel, AllinkBaseTranslatableModel):
    slug_source_field_name = 'title'

    title = TranslatedField(any_language=True)
    lead = TranslatedField()

    preview_image = FilerImageField(
        verbose_name='Preview Image',
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_preview_image',
    )
    sort_order = models.PositiveIntegerField(
        default=0,
        editable=False,
        db_index=True
    )
    header_placeholder = PlaceholderField(
        'dummy_app_header',
        related_name='%(app_label)s_%(class)s_header_placeholder'
    )
    content_placeholder = PlaceholderField(
        'dummy_app_content',
        related_name='%(app_label)s_%(class)s_content_placeholder'
    )

    objects = DummyAppManager()

    class Meta:
        app_label = 'dummy_app'
        ordering = ('sort_order',)
        verbose_name = 'DummyApp'
        verbose_name_plural = 'DummyApp'


class DummyAppTranslation(AllinkBaseTranslatedFieldsModel):
    master = models.ForeignKey(
        'dummy_app.DummyApp',
        on_delete=models.CASCADE,
        related_name='translations',
        null=True
    )

    title = models.CharField(
        max_length=255
    )
    lead = HTMLField(
        _('Lead Text'),
        help_text='Teaser text that in some cases is used in the list view and/or in the detail view.',
        blank=True,
        null=True,
    )

    class Meta:
        app_label = 'dummy_app'


class DummyAppAppContentPlugin(AllinkBaseAppContentPlugin):
    data_model = DummyApp

    manual_entries = SortedM2MModelField(
        'dummy_app.DummyApp',
        blank=True,
        help_text=('Select and arrange specific entries, or, leave blank to select all. (If '
                   'manual entries are selected the category filtering will be applied as well.)')
    )
    apphook_page = PageField(
        verbose_name='Apphook Page',
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        help_text='If provided, this Apphook-Page will be used to generate the detail link.',
    )

    class Meta:
        app_label = 'dummy_app'



# Generated by Django 2.1.8 on 2019-07-02 17:18

import allink_core.core.models.fields
import allink_core.core_apps.allink_button_link.model_fields
import cms.models.fields
from django.conf import settings
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import djangocms_attributes_field.fields
import filer.fields.file
import filer.fields.image


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filer', '0011_auto_20190418_0137'),
        ('cms', '0022_auto_20180620_1551'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkButtonLinkContainerPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_button_link_allinkbuttonlinkcontainerplugin', serialize=False, to='cms.CMSPlugin')),
                ('alignment_horizontal_desktop', models.CharField(blank=True, choices=[('left', 'Left'), ('center', 'Center'), ('right', 'Right')], help_text='This option overrides the projects default alignment for desktop. (Usually "left")', max_length=50, null=True, verbose_name='Alignment horizontal desktop')),
                ('alignment_horizontal_mobile', models.CharField(blank=True, choices=[('left', 'Left'), ('center', 'Center'), ('right', 'Right')], help_text='This option overrides the projects default alignment for mobile. (Usually "left")', max_length=50, null=True, verbose_name='Alignment horizontal mobile')),
                ('project_css_classes', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AllinkButtonLinkPlugin',
            fields=[
                ('link_object_id', models.IntegerField(help_text='To which object directs the url.', null=True)),
                ('link_model', models.CharField(help_text='Dotted Path to referenced Model', max_length=300, null=True)),
                ('link_url_name', models.CharField(help_text='Name of the App-URL to use.', max_length=64, null=True)),
                ('link_url_kwargs', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, help_text='Keyword arguments used to reverse url.', null=True, size=None)),
                ('link_url', models.URLField(blank=True, default='', help_text='Provide a valid URL to an external website.', max_length=500, verbose_name='External link')),
                ('link_mailto', models.EmailField(blank=True, max_length=255, null=True, verbose_name='Email address')),
                ('link_phone', models.CharField(blank=True, max_length=255, null=True, verbose_name='Phone')),
                ('link_anchor', models.CharField(blank=True, help_text='Appends the value only after the internal or external link.<br>Do <strong>not</strong> include a preceding "#" symbol.', max_length=255, verbose_name='Anchor')),
                ('link_target', models.IntegerField(blank=True, choices=[(1, 'New window'), (2, 'Softpage (large)'), (3, 'Softpage (small)'), (4, 'Lightbox (Forms)'), (5, 'Lightbox (Image)'), (6, 'Lightbox (Default)')], null=True, verbose_name='Link Target')),
                ('link_special', models.CharField(blank=True, max_length=255, null=True, verbose_name='Special Links')),
                ('link_attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('template', models.CharField(choices=[('default_link', 'Internal/External'), ('file_link', 'File (Download)'), ('image_link', 'Image'), ('phone_link', 'Phone'), ('email_link', 'Email'), ('form_link', 'Form'), ('video_embedded_link', 'Video (Embedded)'), ('video_file_link', 'Video (File)')], default='default_link', help_text='Choose a link type in order to display its options below.', max_length=50, verbose_name='Link type')),
                ('label', models.CharField(blank=True, default='', max_length=255, verbose_name='Link text')),
                ('type', allink_core.core_apps.allink_button_link.model_fields.LinkOrButton(default='lnk', max_length=255, verbose_name='Display type')),
                ('btn_context', allink_core.core_apps.allink_button_link.model_fields.Context(choices=[('default', 'Default'), ('primary', 'Primary'), ('success', 'Success'), ('info', 'Info'), ('warning', 'Warning'), ('danger', 'Danger'), ('link', 'Link')], default='default', max_length=255, verbose_name='Variation')),
                ('btn_size', allink_core.core_apps.allink_button_link.model_fields.Size(blank=True, default='md', max_length=255, verbose_name='Size')),
                ('btn_block', models.BooleanField(default=False, verbose_name='Block')),
                ('txt_context', allink_core.core_apps.allink_button_link.model_fields.Context(blank=True, choices=[('', 'Default'), ('primary', 'Primary'), ('success', 'Success'), ('info', 'Info'), ('warning', 'Warning'), ('danger', 'Danger'), ('muted ', 'Muted')], default='', max_length=255, verbose_name='Context')),
                ('icon_left', allink_core.core.models.fields.Icon(blank=True, default='', max_length=255, verbose_name='Icon left')),
                ('icon_right', allink_core.core.models.fields.Icon(blank=True, default='', max_length=255, verbose_name='Icon right')),
                ('email_subject', models.CharField(blank=True, default='', max_length=255, verbose_name='Subject')),
                ('email_body_text', models.TextField(blank=True, default='', verbose_name='Body Text')),
                ('send_internal_mail', models.BooleanField(default=False, help_text='Send confirmation mail to defined internal e-mail addresses.', verbose_name='Send internal e-mail')),
                ('internal_email_addresses', django.contrib.postgres.fields.ArrayField(base_field=models.EmailField(blank=True, max_length=254, null=True), blank=True, null=True, size=None, verbose_name='Internal e-mail addresses')),
                ('from_email_address', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Sender e-mail address')),
                ('send_external_mail', models.BooleanField(default=False, help_text='Send confirmation mail to customer.', verbose_name='Send external e-mail')),
                ('thank_you_text', models.TextField(blank=True, help_text='This text will be shown, after form completion.', null=True, verbose_name='Thank you text')),
                ('label_layout', models.CharField(choices=[('stacked', 'Stacked with fields'), ('side_by_side', 'Side by side with fields'), ('placeholder', 'As placeholders')], default='stacked', max_length=15, verbose_name='Display labels')),
                ('project_css_classes', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, null=True, size=None)),
                ('video_id', models.CharField(blank=True, help_text='Only provide the ID. The correct URL will automatically be generated.<br><br>YouTube: https://www.youtube.com/watch?v=<strong>12345678</strong> (the ID is <strong>12345678</strong>)<br>Vimeo: https://vimeo.com/<strong>12345678</strong> (the ID is <strong>12345678</strong>)', max_length=255, null=True, verbose_name='Video ID')),
                ('video_service', models.CharField(blank=True, choices=[('youtube', 'Youtube'), ('vimeo', 'Vimeo')], max_length=50, null=True, verbose_name='Video Service')),
                ('ratio', models.CharField(blank=True, max_length=50, null=True, verbose_name='Ratio')),
                ('video_muted_enabled', models.BooleanField(default=True, help_text='Caution: Autoplaying videos with audio is not recommended. Use wisely.', verbose_name='Muted')),
                ('auto_start_enabled', models.BooleanField(default=False, help_text='<strong>Important:</strong> Autoplaying videos with audio is not recommended. Use wisely. ', verbose_name='Autostart')),
                ('allow_fullscreen_enabled', models.BooleanField(default=True, verbose_name='Allow fullscreen')),
                ('data_modal_escape_close_enabled', models.BooleanField(default=True, verbose_name='Escape key closes modal')),
                ('data_modal_overlay_close_enabled', models.BooleanField(default=True, verbose_name='Click on overlay closes modal')),
                ('data_modal_button_close_enabled', models.BooleanField(default=True, verbose_name='Display close button')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_button_link_allinkbuttonlinkplugin', serialize=False, to='cms.CMSPlugin')),
                ('link_apphook_page', cms.models.fields.PageField(help_text='If provided, overrides the external link.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='allink_button_link_allinkbuttonlinkplugin_app_legacy_redirects', to='cms.Page', verbose_name='New Apphook-Page')),
                ('link_file', filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='filer.File', verbose_name='file')),
                ('link_page', cms.models.fields.PageField(help_text='If provided, overrides the external link and New Apphook-Page.', null=True, on_delete=django.db.models.deletion.PROTECT, to='cms.Page', verbose_name='New Page')),
                ('video_file', filer.fields.file.FilerFileField(blank=True, help_text='Recommended video settings:<br><br>Format: mp4<br>Codec: H.264<br>Target Bitrate: 2 (video loads quick and runs smooth)<br>Audio: Not recommended (no audio = smaller file size and less annoyed visitors)<br>File size: Dependent of video length. Generally speaking: Less is more.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='allink_button_link_allinkbuttonlinkplugin_video_file', to='filer.File', verbose_name='Video File')),
                ('video_poster_image', filer.fields.image.FilerImageField(blank=True, help_text='Image that is being displayed while the video is loading. Ideally the very first frame of the video is used, in order to make the transition as smooth as possible.<br><br><strong>Imoprtant:</strong> Make sure the aspect ratio of the image is <strong>exactly the same</strong> as the video, otherwise the video height will shrink or grow when the playback starts.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='allink_button_link_allinkbuttonlinkplugin_video_poster_image', to=settings.FILER_IMAGE_MODEL, verbose_name='Video Start Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin', models.Model),
        ),
    ]



# -*- coding: utf-8 -*-
from django.db import models

from cms.models.pluginmodel import CMSPlugin

from allink_core.core.models.fields import CMSPluginField
from allink_core.core.models import AllinkInternalLinkFieldsModel
from allink_core.core.utils import get_additional_templates
from allink_core.core.models.fields_model import AllinkTeaserFieldsModel, AllinkTeaserTranslatedFieldsModel


class AllinkTeaserPlugin(AllinkInternalLinkFieldsModel, AllinkTeaserFieldsModel, AllinkTeaserTranslatedFieldsModel,
                         CMSPlugin):
    template = models.CharField(
        'Template',
        max_length=50
    )

    softpage_enabled = models.BooleanField(
        'Show detailed information in Softpage',
        help_text='If checked, the detail view of an entry will be displayed in a "softpage".'
                  ' Otherwise the page will be reloaded.',
        default=False
    )

    cmsplugin_ptr = CMSPluginField()

    def __str__(self):
        return '{}'.format(self.link_object)

    @classmethod
    def get_templates(cls):
        templates = ()
        for x, y in get_additional_templates('TEASER'):
            templates += ((x, y),)
        return templates



# Generated by Django 2.1.8 on 2019-07-02 17:18

from django.conf import settings
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields
import filer.fields.folder
import filer.fields.image


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filer', '0011_auto_20190418_0137'),
        ('cms', '0022_auto_20180620_1551'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkGalleryImagePlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_gallery_allinkgalleryimageplugin', serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Title')),
                ('text', djangocms_text_ckeditor.fields.HTMLField(blank=True, null=True, verbose_name='Text')),
                ('template', models.CharField(help_text='Choose a template.', max_length=50, verbose_name='Template')),
                ('ratio', models.CharField(blank=True, max_length=50, null=True, verbose_name='Ratio')),
                ('image', filer.fields.image.FilerImageField(on_delete=django.db.models.deletion.PROTECT, to=settings.FILER_IMAGE_MODEL, verbose_name='Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AllinkGalleryPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_gallery_allinkgalleryplugin', serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(help_text='Choose a template.', max_length=50, verbose_name='Template')),
                ('ratio', models.CharField(blank=True, max_length=50, null=True, verbose_name='Ratio')),
                ('fullscreen_enabled', models.BooleanField(default=True, help_text='This option enables a fullscreen button for this gallery.', verbose_name='Fullscreen option visible')),
                ('counter_enabled', models.BooleanField(default=False, help_text='This option enables a gallery counter.', verbose_name='Gallery counter visible')),
                ('auto_start_enabled', models.BooleanField(default=True, help_text='This option enables autoplay for this gallery.', verbose_name='Autostart')),
                ('project_css_classes', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, null=True, size=None)),
                ('folder', filer.fields.folder.FilerFolderField(blank=True, help_text="All Images (.png, .gif, .jpg, .jpeg) will be used in gallery. If a folder is specified, the child plugin won't be rendered.", null=True, on_delete=django.db.models.deletion.PROTECT, to='filer.Folder')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
import mandrill
from django import forms
from django.conf import settings
from django.utils import translation
from django.utils.translation import ugettext as _
from django.urls import reverse
from django.utils.dateformat import DateFormat
from raven import Client

from allink_core.core.utils import base_url
from allink_core.core.utils import get_display
from allink_core.core_apps.allink_mandrill.config import MandrillConfig
from allink_core.core_apps.allink_mandrill.helpers import check_result_status


class AllinkMandrillEmailBase:
    """
    Sends emails with mandrill.
    The only thing you have to specify is "template_name" the rest of the attributes are optional.

    have a look at the ./README.md for a basic guide on how to implement this class.
    """
    # The template can be sent with two types of merge var language settings
    MERGE_LANG_MAILCHIMP = 'mailchimp'  # Notation: *|FIRST_NAME|*
    MAILCHIMP_FORMAT = "*|{}|*"
    MERGE_LANG_HANDELBAR = 'handlebars'  # Notation: {FIRST_NAME}
    HANDELBAR_FORMAT = "{{{}}}"
    MERGE_FORMAT = {
        MERGE_LANG_MAILCHIMP: MAILCHIMP_FORMAT,
        MERGE_LANG_HANDELBAR: HANDELBAR_FORMAT,

    }
    template_name = None
    # Optional attributes
    merge_language = MERGE_LANG_MAILCHIMP
    translated = True
    google_analytics_campaign = None
    view_content_link = True
    # to make this as flexible as possible we added all the settings options of the Mandrill api message to the attrs
    # I have never seen these changed but if necessary you can do this here
    return_path_domain = None
    preserve_recipients = True
    inline_css = True
    auto_html = None
    auto_text = None

    def __init__(self, template_name=None, logging=False, language=None, async=False, send_with_celery=False,
                 track_clicks=True, track_opens=True, *args, **kwargs):
        """
        Args:
            template_name: mandrill template name (if translated=True, do not add the "_de")
            logging: Boolean if logging is turned on or nor if true implement
                def log(self, kwargs**)
        """
        self.config = MandrillConfig()
        self.logging = logging
        if self.translated:
            assert (language, 'You have to provide a language, if you send a translated email.')
            self.language = language
        self.async = async
        self.send_with_celery = send_with_celery
        if template_name:
            self.template_name = template_name
        self.track_opens = track_opens
        self.track_clicks = track_clicks

    @staticmethod
    def create_global_merge_var(template_keyword, content):
        """
        Args:
            template_keyword: Template Keyword used by Mandrill
            content: Content for this Template Keword

        Returns: Formatted object that meets requirements  for Mandrill API
                {'name': template_keyword, 'content': str(content)}
        """
        # JavaScript expects True like this "true" so we need this thingy
        if content == "False":
            return {'name': str(template_keyword), 'content': False}
        elif content == "True":
            return {'name': str(template_keyword), 'content': True}

        return {'name': str(template_keyword), 'content': str(content)}

    @staticmethod
    def create_email_to_entry(email_address, name):
        return {
            'email': email_address,
            'name': name,
            'type': 'to'
        }

    @staticmethod
    def replace_linebreaks_with_html(content):
        r = '<br />'
        return content.replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)

    def build_subject(self):
        raise NotImplementedError

    def build_body(self):
        raise NotImplementedError

    def get_from_mail_address(self):
        raise NotImplementedError

    def fetch_to_email_addresses(self):
        raise NotImplementedError

    def log(self, **kwargs):
        # this is only needed if self.logging==True
        raise NotImplementedError

    def get_language(self):
        return self.language

    def get_template_name(self):
        assert self.template_name
        if not self.translated:
            return self.template_name
        if self.language:
            template_name = '{}_{}'.format(self.template_name, self.get_language())
        return template_name

    def get_from_name(self):
        return self.config.get_default_from_name()

    def get_website_for_metadata(self):
        return self.config.get_site_domain()

    def get_reply_to_email_address(self):
        return self.get_from_mail_address()

    def get_googleanalytics_domains(self):
        return [self.config.get_site_domain(), ]

    def send_transactional_email(self, message, template_content):
        if self.send_with_celery:

            from .tasks import send_transactional_mail_celery

            template_name = self.get_template_name()
            send_transactional_mail_celery.apply_async(args=[template_name, message, template_content], queue='slow')

        else:
            try:
                mandrill_client = mandrill.Mandrill(self.config.apikey)
                result = mandrill_client.messages.send_template(template_name=self.get_template_name(),
                                                                template_content=template_content, message=message,
                                                                async=self.async)
                check_result_status(result)
            except:  # as we just push it to sentry we catch all the errors
                if not hasattr(settings, 'RAVEN_CONFIG'):
                    raise
                client = Client(settings.RAVEN_CONFIG.get('dsn'))
                client.captureException()
                # sentry is not configured on localhost
                if not settings.RAVEN_CONFIG.get('dsn'):
                    raise

    def send_mail(self):
        """
        This should never throw an exception. Instead it fails silently and sends exceptions to sentry.
        (So should never have to wrap your .send_mail() call in a try/ except.
        for a better handling of failed emails you should implement an appropriate log
        """
        if self.translated:
            prev_language = translation.get_language()
            translation.activate(self.language)
        try:
            template_content = [{}]
            # we can only send, when we have a email address
            if self.fetch_to_email_addresses() and not self.fetch_to_email_addresses() == '':
                message = {
                    'auto_html': self.auto_html,
                    'auto_text': self.auto_text,
                    'from_email': str(self._dev_mode_safe_from_adress()),
                    'from_name': str(self.get_from_name()),
                    'global_merge_vars': self.build_body(),
                    'google_analytics_campaign': self.google_analytics_campaign,
                    'google_analytics_domains': self.get_googleanalytics_domains(),
                    'headers': {'Reply-To': self.get_reply_to_email_address()},
                    'inline_css': self.inline_css,
                    'merge_language': self.merge_language,
                    'metadata': {'website': self.get_website_for_metadata()},
                    'preserve_recipients': self.preserve_recipients,
                    'return_path_domain': self.return_path_domain,
                    'subject': str(self.build_subject()),
                    'to': self._dev_mode_safe_to_adresses(),
                    'track_clicks': self.track_clicks,
                    'track_opens': self.track_opens,
                    'view_content_link': self.view_content_link,
                }

                self.send_transactional_email(message=message, template_content=template_content)
                # if logging is enabled the function log will be called. You will have to implement it
                if self.logging:
                    self.log(to=message['to'],
                             subject=message['subject'],
                             form=message['from_email'],
                             body_vars=message['global_merge_vars'],
                             full_message=message)
        finally:
            if self.translated:
                translation.activate(prev_language)

    def _fetch_to_email_adresses_dev_mode(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        return [self.create_email_to_entry(email, 'allink Test')
                for email in settings.ALLINK_MANDRILL_DEV_MODE_TO_EMAIL_ADDRESSES]

    def _get_from_email_adress_dev_mode(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        return settings.ALLINK_MANDRILL_DEV_MODE_FROM_EMAIL_ADDRESS

    def _dev_mode_safe_from_adress(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        if settings.ALLINK_MANDRILL_DEV_MODE:
            return self._get_from_email_adress_dev_mode()
        else:
            return self.get_from_mail_address()

    def _dev_mode_safe_to_adresses(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        if settings.ALLINK_MANDRILL_DEV_MODE:
            return self._fetch_to_email_adresses_dev_mode()
        else:
            return self.fetch_to_email_addresses()


class AllinkMandrillFormEmail(AllinkMandrillEmailBase):
    """
    Takes an additional parameter "form"
    - adds all form fields to the merge vars
    - adds a merge var "form_fields" (this will list all available form fields.
      and can be used in the mailchimp template as a reference)

    APPEND_OPTIONAL_FIELDS = False
      all empty fields won't be send to mandrill

    example implementation:
    class SomeEmail(AllinkMandrillFormEmail):
        template_name = 'entra_living_signup_confirmation'
        google_analytics_campaign = template_name

        def fetch_to_email_addresses(self):
            return [
                self.create_email_to_entry(
                    self.form.data.get('email'),
                    ''.format(self.form.fields.get('last_name'), self.form.fields.get('first_name'))
                ),
            ]

    SomeEmail(form=form, language=get_language()).send_mail()
    """
    APPEND_OPTIONAL_FIELDS = True

    def __init__(self, form, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.form = form

    def build_body(self):
        form_fields_merge_vars = self.get_merge_vars_from_form(self.form)

        body = form_fields_merge_vars
        body.append(self.create_global_merge_var('submitted_form_str',
                                                 self.get_submitted_form_str(
                                                     self.form, form_fields_merge_vars)))
        body.append(self.create_global_merge_var('submitted_form_url', self.get_submitted_form_url(self.form)))
        body.append((self.create_global_merge_var('list_of_available_merge_vars',
                                                  self.get_list_of_available_merge_vars(self.form))))
        return body

    def get_merge_vars_from_form(self, form):
        """
        returns a list with all the form fields as a merge vars dict
            [('name': 'foo', 'content': 'bar'), ..]
        """
        form_fields_merge_vars = []
        for field in form.fields:
            if form.data.get(field):
                if type(form.fields.get(field)) == forms.MultipleChoiceField:
                    selected_choices = []
                    for option in form.fields.get(field).choices:
                        name, title = option
                        if name in form.cleaned_data.get(field):
                            selected_choices.append(translation.gettext(title))
                    form_fields_merge_vars.append(self.create_global_merge_var(field, '<br>'.join(selected_choices)))
                elif type(form.fields.get(field)) == forms.DateField:
                    # date field
                    date = DateFormat(form.cleaned_data.get(field)).format('j. F Y')
                    form_fields_merge_vars.append(self.create_global_merge_var(field, date))
                elif hasattr(form.fields.get(field), 'choices'):
                    # model field choices
                    form_fields_merge_vars.append(self.create_global_merge_var(field, get_display(
                        form.data.get(field), form.fields.get(field).choices)))
                elif hasattr(form.fields.get(field).widget, 'choices'):
                    # form field choices
                    form_fields_merge_vars.append(
                        self.create_global_merge_var(field, get_display(form.data.get(field), form.fields.get(
                            field).widget.choices)))
                elif type(form.fields.get(field)) == forms.BooleanField:
                    # checkbox
                    form_fields_merge_vars.append(
                        self.create_global_merge_var(field, _('Yes') if True else _('No')))
                # TODO form.data.get(field) is empty
                # elif type(form.fields.get(field)) == forms.FileField:
                #     # file field
                #     file_link_str = '{}{}'.format(base_url(), form.cleaned_data.get(field).file.url)
                #     form_fields_merge_vars.append(
                #         self.create_global_merge_var(field, file_link_str))
                else:
                    form_fields_merge_vars.append(self.create_global_merge_var(field, form.data.get(field)))
            else:
                # form field empty (optional)
                if self.APPEND_OPTIONAL_FIELDS:
                    form_fields_merge_vars.append(self.create_global_merge_var(field, '—'))
        return form_fields_merge_vars

    def get_submitted_form_str(self, form, form_fields_merge_vars):
        """
        for convenience we add a merge var 'submitted_form'
        (can be used to display all the form fields at once, including labels and linebreaks)
            [('name': 'submitted_form', 'content': '<<string with all form fields and labels separated with <br>'), ..]
        """
        submitted_form_str = '<br>'.join(
            ['{}:<br>{}<br>'.format(form.fields.get(merge_var.get('name')).label,
                                    merge_var.get('content')) for merge_var in form_fields_merge_vars])
        return submitted_form_str

    def get_submitted_form_url(self, form):
        """
        we add a link to the submitted form (url to django admin)
            [('name': 'submitted_form_url', 'content': '<<absolute url to django admin>>'), ..]
        """
        submitted_form_url = '{}{}'.format(base_url(),
                                           reverse('admin:%s_%s_change' % (form.instance._meta.app_label,
                                                                           form.instance._meta.model_name),
                                                   args=[form.instance.id]))
        return submitted_form_url

    def get_list_of_available_merge_vars(self, form):
        """
        we add a merge var 'list_of_available_merge_vars' (can be used as a reference in mailchimp templates)
            [('name': 'form_fields', 'content': '<<string with all form fields separated with
            <br> and formatted in the merge_language>>'), ..]
        """
        form_fields_combined = '<br>'.join(
            [self.MERGE_FORMAT[self.merge_language].format(field) for field in form.fields]
        )
        return form_fields_combined


class AllinkMandrillFormPluginEmail(AllinkMandrillFormEmail):
    """
    Takes parameters "form" and "plugin". The Plugin needs to define at least the fields in AllinkBaseFormPlugin.
    The plugin defines the email subject, sender email address and the internal recipients.

    example implementation:
    class SomeInternalEmail(AllinkMandrillFormPluginEmail):
        template_name = 'entra_living_signup_confirmation'
        google_analytics_campaign = template_name

        def fetch_to_email_addresses(self):
            return [self.create_email_to_entry(email, self.config.get_default_from_name()) for email in self.plugin.internal_recipients]

    SomeInternalEmail(form=form, plugin=plugin, language=get_language()).send_mail()
    """

    def __init__(self, plugin, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plugin = plugin

    def build_subject(self):
        return self.plugin.email_subject

    def get_from_mail_address(self):
        return self.plugin.from_email_address



# -*- coding: utf-8 -*-

from django.contrib import admin
from django import forms
from django.conf import settings
from parler.admin import TranslatableAdmin
from django.http import HttpResponseRedirect

from treebeard.admin import TreeAdmin
from treebeard.forms import movenodeform_factory, MoveNodeForm

from parler.forms import TranslatableModelForm

from allink_core.core.utils import get_additional_choices
from allink_core.core_apps.allink_categories.models import AllinkCategory


class AllinkCategoryForm(TranslatableModelForm, MoveNodeForm):
    model_names = forms.MultipleChoiceField(
        label='Project app',
        help_text=('Please specify the app which uses this categories. All apps specified in parent category '
                   'are automatically added.'),
        choices=settings.PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        required=False
    )

    class Meta():
        model = AllinkCategory
        fields = ('name', 'identifier', 'slug', 'model_names')

    def __init__(self, *args, **kwargs):
        super(AllinkCategoryForm, self).__init__(*args, **kwargs)
        # hide model_names when
        if self.instance and not self.instance.is_root():
            self.fields['model_names'].widget = forms.MultipleHiddenInput()

        if get_additional_choices('PROJECT_CATEGORY_IDENTIFIERS'):
            self.fields['identifier'] = forms.ChoiceField(
                label='Identifier',
                help_text='Identifier used for backward reference on a app model. (e.g display category name '
                            'on People app, e.g Marketing)',
                choices=get_additional_choices('PROJECT_CATEGORY_IDENTIFIERS', blank=True),
                required=False,
            )


@admin.register(AllinkCategory)
class AllinkCategoryAdmin(TranslatableAdmin, TreeAdmin):
    form = movenodeform_factory(AllinkCategory, form=AllinkCategoryForm)
    list_display = ('name', )
    # fields = ('name',  'slug', 'model_names')

    def response_add(self, request, obj, post_url_continue=None):
        if obj.is_root():
            return HttpResponseRedirect("../%s" % obj.id)
        else:
            return super(AllinkCategoryAdmin, self).response_add(request, obj, post_url_continue=post_url_continue)



import json
from django.urls import reverse
from cms.test_utils.testcases import CMSTestCase
from django.test.testcases import TestCase
from allink_core.core.test import CategoriesMixin, DataModelMixin, PageApphookMixin, AllinkAppContenPluginMixin
from ..cms_apps import NewsApphook
from ..models import News
from ..views import NewsPluginLoadMore
from ..cms_plugins import CMSNewsAppContentPlugin
from .factories import NewsFactory


class NewsDetailViewTestCase(DataModelMixin, PageApphookMixin, CMSTestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'
    apphook_object = NewsApphook
    data_model_factory = NewsFactory

    def setUp(self):
        super().setUp()
        self.admin_user = self.get_superuser()

    def test_detail_when_inactive_404(self):
        inactive_entry = NewsFactory(status=News.INACTIVE)
        detail_url = reverse('{0}:detail'.format(self.namespace), kwargs={'slug': inactive_entry.slug})
        response = self.client.get(detail_url)
        self.assertEqual(response.status_code, 404)

    def test_detail_when_inactive_and_staff_user_200(self):
        inactive_entry = NewsFactory(status=News.INACTIVE)
        detail_url = reverse('{0}:detail'.format(self.namespace), kwargs={'slug': inactive_entry.slug})
        with self.login_user_context(self.admin_user):
            response = self.client.get(detail_url)
            self.assertEqual(response.status_code, 200)

    def test_detail_when_active_200(self):
        detail_url = reverse('{0}:detail'.format(self.namespace), kwargs={'slug': self.entry_1.slug})
        response = self.client.get(detail_url)
        self.assertEqual(response.status_code, 200)

    # def test_detail_get_template_names(self):
        # TODO 'get_template_names' method is already pretty voodoo. We should make this more explicit.
        # template_names = NewsDetail().get_template_names()

    # def test_detail_when_ajax_call(self):


class NewsPluginLoadMoreViewTestCaseApp(CategoriesMixin, DataModelMixin, AllinkAppContenPluginMixin, TestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'
    plugin_class = CMSNewsAppContentPlugin
    apphook_object = NewsApphook
    data_model_factory = NewsFactory
    load_more_view = NewsPluginLoadMore

    def test_more_without_plugin_id(self):
        more_url = '{}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}))
        response = self.client.get(more_url)
        self.assertEqual(response.status_code, 404)

    def test_more_without_plugin_in_db(self):
        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), 100)
        response = self.client.get(more_url)
        self.assertEqual(response.status_code, 404)

    def test_more_with_plugin_id(self):
        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(response.status_code, 200)

    # def test_set_category_id(self):
    #     """"""
    #     pass

    def test_contenty_type_is_json(self):
        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(response['Content-Type'], 'application/json')

    def test_json_no_result_false(self):
        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(json.loads(response.content).get('no_results'), False)

    def test_json_no_result_true(self):
        self.plugin_model_instance.categories.set([self.category_1,])
        self.plugin_model_instance.save()

        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(json.loads(response.content).get('no_results'), True)

    def test_json_with_pagination_no_next_page_url(self):
        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertIsNone(json.loads(response.content).get('next_page_url'))

    def test_json_with_pagination_no_next_page_url_last_page(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD
        self.plugin_model_instance.save()

        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 3}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertIsNone(json.loads(response.content).get('next_page_url'))

    def test_json_with_pagination_next_page_url(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD
        self.plugin_model_instance.save()

        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        next_page_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 2}), self.plugin_model_instance.id)
        self.assertEqual(json.loads(response.content).get('next_page_url'), next_page_url)

    def test_rendered_html_class_appended(self):
        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(json.loads(response.content).get('rendered_content').count('appended'), 5)

    def test_rendered_html_pagination_first_page(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD
        self.plugin_model_instance.save()

        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 1}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(json.loads(response.content).get('rendered_content').count('>news entry'), 2)

    def test_rendered_html_pagination_second_page(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD
        self.plugin_model_instance.save()

        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 2}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(json.loads(response.content).get('rendered_content').count('>news entry'), 2)

    def test_rendered_html_pagination_last_page(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD
        self.plugin_model_instance.save()

        more_url = '{}?plugin_id={}'.format(reverse('{0}:more'.format(self.namespace), kwargs={'page': 3}), self.plugin_model_instance.id)
        response = self.client.get(more_url)
        self.assertEqual(json.loads(response.content).get('rendered_content').count('>news entry'), 1)

    def test_get_queryset_default_all(self):
        """ TODO: will be obsolete after refactoring, because tested on plugin model """
        load_more_view = self.get_load_more_view()

        queryset = load_more_view.get_queryset()
        self.assertEqual(queryset.count(), 5)

    def test_get_queryset_default_filtered_by_category(self):
        """ Load more with a specific category. Default queryset -> ?plugin_id=1&category=2 """
        self.entry_1.categories.set([self.category_1])
        self.entry_2.categories.set([self.category_1])
        self.entry_3.categories.set([self.category_2])

        load_more_view = self.get_load_more_view()
        load_more_view.category_id = self.category_2.id  # this would be set by GET['category] = 2 in get method.
        load_more_view.category = self.category_2  # this would be set by GET['category] = 2 in get method.

        queryset = load_more_view.get_queryset()
        self.assertEqual(queryset.count(), 1)

    def test_get_queryset_manual_entries(self):
        """ TODO: will be obsolete after refactoring, because tested on plugin model """
        self.plugin_model_instance.manual_entries.set([self.entry_1, self.entry_2])

        load_more_view = self.get_load_more_view()

        queryset = load_more_view.get_queryset()
        self.assertEqual(queryset.count(), 2)

    def test_get_queryset_manual_entries_filtered_by_category(self):
        """ Load more with a specific category. Entries are selected manually. -> ?plugin_id=1&category=2 """
        self.entry_1.categories.set([self.category_1])
        self.entry_2.categories.set([self.category_2])
        self.plugin_model_instance.manual_entries.set([self.entry_1, self.entry_2])

        load_more_view = self.get_load_more_view()
        load_more_view.category_id = self.category_2.id  # this would be set by GET['category] = 2 in get method.
        load_more_view.category = self.category_2  # this would be set by GET['category] = 2 in get method.

        queryset = load_more_view.get_queryset()
        self.assertEqual(queryset.count(), 1)

    def test_get_context_data_no_pagination(self):
        load_more_view = self.get_load_more_view()
        load_more_view.object_list = load_more_view.get_queryset() # this would be set in get method.

        context = load_more_view.get_context_data()
        self.assertIsNone(context['paginator'])
        self.assertIsNone(context['page_obj'])
        self.assertFalse(context['is_paginated'])
        self.assertEqual(context['object_list'].count(), 5)

    def test_get_context_data_with_pagination_load_more(self):
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD
        self.plugin_model_instance.save()

        load_more_view = self.get_load_more_view()
        load_more_view.kwargs = {'page': 1}
        load_more_view.object_list = load_more_view.get_queryset() # this would be set in get method.

        context = load_more_view.get_context_data()
        self.assertIsNotNone(context['paginator'])
        self.assertEqual(context['page_obj'].object_list.count(), 2)
        self.assertTrue(context['is_paginated'])
        self.assertEqual(context['object_list'].count(), 2)

    def test_get_context_data_with_pagination_load_rest_first_page(self):
        """
        We used a quiet hacky way to use the paginator to load all the remaining items on the second page.
        paginator and page_object are not set correctly. We let the wrong values here!

        For displaying the "Load All" Button we use all information on the queryset. This might be hacky, but works.

        With LOAD_REST we use .paginated_by from the plugin on the first page. The second page should display all the remaining items.
        !! This is still broken, but we never noticed, because this view is always called with page=2.
        """
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD_REST
        self.plugin_model_instance.save()

        load_more_view = self.get_load_more_view()
        load_more_view.kwargs = {'page': 1}
        load_more_view.object_list = load_more_view.get_queryset() # this would be set in get method.

        context = load_more_view.get_context_data()
        self.assertEqual(context['paginator'].num_pages, 3)  # this is actually wrong, should be 2, as we want all the entries on the second page
        self.assertEqual(context['page_obj'].object_list.count(), 1)  # this is actually wrong, should be 3, as we want all the entries on the second page
        self.assertTrue(context['is_paginated'])
        self.assertEqual(context['object_list'].count(), 3)

    def test_get_context_data_with_pagination_load_rest_second_page(self):
        """
        We used a quiet hacky way to use the paginator to load all the remaining items on the second page.
        paginator and page_object are not set correctly. We let the wrong values here!

        For displaying the "Load All" Button we use all information on the queryset. This might be hacky, but works.
        """
        self.plugin_model_instance.paginated_by = 2
        self.plugin_model_instance.pagination_type = self.plugin_model_instance.LOAD_REST
        self.plugin_model_instance.save()

        load_more_view = self.get_load_more_view()
        load_more_view.kwargs = {'page': 2}
        load_more_view.object_list = load_more_view.get_queryset() # this would be set in get method.

        context = load_more_view.get_context_data()
        self.assertEqual(context['paginator'].num_pages, 3)  # this is actually wrong, should be 2, as we want all the entries on the second page
        self.assertEqual(context['page_obj'].object_list.count(), 1)  # this is actually wrong, should be 3. but is now set to the actual last page of the original paginator where the count 1
        self.assertFalse(context['page_obj'].has_next())
        self.assertTrue(context['is_paginated'])
        self.assertEqual(context['object_list'].count(), 3)







# Generated by Django 2.1.10 on 2019-07-23 11:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_legacy_redirect', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinklegacylink',
            name='language',
            field=models.CharField(choices=[('en', 'English'), ('de', 'German')], default=('en', 'English'), max_length=200, null=True, verbose_name='Language'),
        ),
    ]



# Generated by Django 2.1.8 on 2019-07-02 17:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkSEOAccordion',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_seo_accordion_allinkseoaccordion', serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AllinkSEOAccordionContainerPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_seo_accordion_allinkseoaccordioncontainerplugin', serialize=False, to='cms.CMSPlugin')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from cms.plugin_pool import plugin_pool
from allink_core.core.cms_plugins import CMSAllinkBaseAppContentPlugin

from .models import DummyAppAppContentPlugin


@plugin_pool.register_plugin
class CMSDummyAppAppContentPlugin(CMSAllinkBaseAppContentPlugin):
    """
    model:
    - where to store plugin instances

    name:
    - name of the plugin
    """
    model = DummyAppAppContentPlugin
    name = model.data_model._meta.verbose_name_plural
